# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2018-2020, 2023 Dominik Kriegner <dominik.kriegner@gmail.com>

"""
implement convenience functions to define Heusler materials.
"""

from . import elements
from .material import Crystal
from .spacegrouplattice import SGLattice

__all__ = [
    "FullHeuslerCubic225",
    "FullHeuslerCubic225_A2",
    "FullHeuslerCubic225_B2",
    "FullHeuslerCubic225_DO3",
    "HeuslerHexagonal194",
    "HeuslerTetragonal119",
    "HeuslerTetragonal139",
    "InverseHeuslerCubic216",
]


def _check_elements(*elem):
    ret = []
    for el in elem:
        if isinstance(el, str):
            ret.append(getattr(elements, el))
        else:
            ret.append(el)
    return ret


def FullHeuslerCubic225(X, Y, Z, a, biso=(0, 0, 0), occ=(1, 1, 1)):
    """
    Full Heusler structure with formula X2YZ.
    Strukturberichte symbol L2_1; space group Fm-3m (225)

    Parameters
    ----------
    X, Y, Z :   str or Element
        elements
    a :         float
        cubic lattice parameter in angstrom
    biso :      list of floats, optional
        Debye Waller factors for X, Y, Z elements
    occ :       list of floats, optional
        occupation numbers for the elements X, Y, Z

    Returns
    -------
    Crystal
        Crystal describing the Heusler material
    """
    x, y, z = _check_elements(X, Y, Z)
    return Crystal(
        f"{x.basename}2{y.basename}{z.basename}",
        SGLattice(
            225, a, atoms=[x, y, z], pos=["8c", "4a", "4b"], b=biso, occ=occ
        ),
    )


def FullHeuslerCubic225_B2(X, Y, Z, a, b2dis, biso=(0, 0, 0), occ=(1, 1, 1)):
    """
    Full Heusler structure with formula X2YZ.
    Strukturberichte symbol L2_1; space group Fm-3m (225) with B2-type (CsCl)
    disorder

    Parameters
    ----------
    X, Y, Z :   str or Element
        elements
    a :         float
        cubic lattice parameter in angstrom
    b2dis :     float
        amount of B2-type disorder (0: fully ordered, 1: fully disordered)
    biso :      list of floats, optional
        Debye Waller factors for X, Y, Z elements
    occ :       list of floats, optional
        occupation numbers for the elements X, Y, Z

    Returns
    -------
    Crystal
        Crystal describing the Heusler material
    """
    x, y, z = _check_elements(X, Y, Z)
    return Crystal(
        f"{x.basename}2{y.basename}{z.basename}",
        SGLattice(
            225,
            a,
            atoms=[x, y, z, y, z],
            pos=["8c", "4a", "4b", "4b", "4a"],
            occ=[
                1 * occ[0],
                (1 - b2dis / 2.0) * occ[1],
                (1 - b2dis / 2.0) * occ[2],
                b2dis / 2.0 * occ[1],
                b2dis / 2.0 * occ[2],
            ],
            b=biso + [biso[1], biso[2]],
        ),
    )


def FullHeuslerCubic225_A2(X, Y, Z, a, a2dis, biso=(0, 0, 0), occ=(1, 1, 1)):
    """
    Full Heusler structure with formula X2YZ.
    Strukturberichte symbol L2_1; space group Fm-3m (225) with A2-type (W)
    disorder

    Parameters
    ----------
    X, Y, Z :   str or Element
        elements
    a :         float
        cubic lattice parameter in angstrom
    a2dis :     float
        amount of A2-type disorder (0: fully ordered, 1: fully disordered)
    biso :      list of floats, optional
        Debye Waller factors for X, Y, Z elements
    occ :       list of floats, optional
        occupation numbers for the elements X, Y, Z

    Returns
    -------
    Crystal
        Crystal describing the Heusler material
    """
    x, y, z = _check_elements(X, Y, Z)
    return Crystal(
        f"{x.basename}2{y.basename}{z.basename}",
        SGLattice(
            225,
            a,
            atoms=[x, x, x, y, y, y, z, z, z],
            pos=["8c", "4a", "4b", "8c", "4a", "4b", "8c", "4a", "4b"],
            occ=[
                (1 - a2dis / 2.0) * occ[0],
                a2dis / 2.0 * occ[0],
                a2dis / 2.0 * occ[0],
                a2dis / 4.0 * occ[1],
                (1 - a2dis * 3.0 / 4.0) * occ[1],
                a2dis / 4.0 * occ[1],
                a2dis / 4.0 * occ[2],
                a2dis / 4.0 * occ[2],
                (1 - a2dis * 3.0 / 4.0) * occ[2],
            ],
            b=[
                biso[0],
            ]
            * 3
            + [
                biso[1],
            ]
            * 3
            + [
                biso[2],
            ]
            * 3,
        ),
    )


def FullHeuslerCubic225_DO3(
    X, Y, Z, a, do3disxy, do3disxz, biso=(0, 0, 0), occ=(1, 1, 1)
):
    """
    Full Heusler structure with formula X2YZ.
    Strukturberichte symbol L2_1; space group Fm-3m (225) with DO_3-type (BiF3)
    disorder, either between atoms X <-> Y or X <-> Z.

    Parameters
    ----------
    X, Y, Z :   str or Element
        elements
    a :         float
        cubic lattice parameter in angstrom
    do3disxy :  float
        amount of DO_3-type disorder between X and Y atoms (0: fully ordered,
        1: fully disordered)
    do3disxz :  float
        amount of DO_3-type disorder between X and Z atoms (0: fully ordered,
        1: fully disordered)
    biso :      list of floats, optional
        Debye Waller factors for X, Y, Z elements
    occ :       list of floats, optional
        occupation numbers for the elements X, Y, Z

    Returns
    -------
    Crystal
        Crystal describing the Heusler material
    """
    x, y, z = _check_elements(X, Y, Z)
    return Crystal(
        f"{x.basename}2{y.basename}{z.basename}",
        SGLattice(
            225,
            a,
            atoms=[x, y, z, x, y, x, z],
            pos=["8c", "4a", "4b", "4a", "8c", "4b", "8c"],
            occ=[
                (1 - do3disxy / 3.0 - do3disxz / 3.0) * occ[0],
                (1 - do3disxy * 2 / 3.0) * occ[1],
                (1 - do3disxz * 2 / 3.0) * occ[2],
                do3disxy * 2 / 3.0 * occ[0],
                do3disxy * 1 / 3.0 * occ[1],
                do3disxz * 2 / 3.0 * occ[0],
                do3disxz * 1 / 3.0 * occ[2],
            ],
            b=biso + [biso[0], biso[1]] + [biso[0], biso[2]],
        ),
    )


def InverseHeuslerCubic216(X, Y, Z, a, biso=(0, 0, 0), occ=(1, 1, 1)):
    """
    Full Heusler structure with formula (XY)X'Z structure;
    space group F-43m (216)

    Parameters
    ----------
    X, Y, Z :   str or Element
        elements
    a :         float
        cubic lattice parameter in angstrom

    Returns
    -------
    Crystal
        Crystal describing the Heusler material
    """
    x, y, z = _check_elements(X, Y, Z)
    return Crystal(
        f"({x.basename}{y.basename}){x.basename}'{z.basename}",
        SGLattice(
            216,
            a,
            atoms=[x, x, y, z],
            pos=["4a", "4d", "4b", "4c"],
            b=[
                biso[0],
            ]
            + biso,
            occ=[
                occ[0],
            ]
            + occ,
        ),
    )


def HeuslerTetragonal139(X, Y, Z, a, c, biso=(0, 0, 0), occ=(1, 1, 1)):
    """
    Tetragonal Heusler structure with formula X2YZ
    space group I4/mmm (139)

    Parameters
    ----------
    X, Y, Z :   str or Element
        elements
    a, c :      float
        tetragonal lattice parameters in angstrom

    Returns
    -------
    Crystal
        Crystal describing the Heusler material
    """
    x, y, z = _check_elements(X, Y, Z)
    return Crystal(
        f"{x.basename}2{y.basename}{z.basename}",
        SGLattice(
            139, a, c, atoms=[x, y, z], pos=["4d", "2b", "2a"], b=biso, occ=occ
        ),
    )


def HeuslerTetragonal119(X, Y, Z, a, c, biso=(0, 0, 0), occ=(1, 1, 1)):
    """
    Tetragonal Heusler structure with formula X2YZ
    space group I-4m2 (119)

    Parameters
    ----------
    X, Y, Z :   str or Element
        elements
    a, c :      float
        tetragonal lattice parameters in angstrom

    Returns
    -------
    Crystal
        Crystal describing the Heusler material
    """
    x, y, z = _check_elements(X, Y, Z)
    return Crystal(
        f"{x.basename}2{y.basename}{z.basename}",
        SGLattice(
            119,
            a,
            c,
            atoms=[x, x, y, z],
            pos=["2b", "2c", "2d", "2a"],
            b=[
                biso[0],
            ]
            + biso,
            occ=[
                occ[0],
            ]
            + occ,
        ),
    )


def HeuslerHexagonal194(X, Y, Z, a, c, biso=(0, 0, 0), occ=(1, 1, 1)):
    """
    Hexagonal Heusler structure with formula XYZ
    space group P63/mmc (194)

    Parameters
    ----------
    X, Y, Z :   str or Element
        elements
    a, c :      float
        hexagonal lattice parameters in angstrom

    Returns
    -------
    Crystal
        Crystal describing the Heusler material
    """
    x, y, z = _check_elements(X, Y, Z)
    return Crystal(
        f"{x.basename}{y.basename}{z.basename}",
        SGLattice(
            194, a, c, atoms=[x, y, z], pos=["2a", "2c", "2d"], b=biso, occ=occ
        ),
    )



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2013-2020 Dominik Kriegner <dominik.kriegner@gmail.com>

import unittest

import numpy
import xrayutilities as xu


class TestQConversion(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.mat = xu.materials.Si
        cls.hxrd = xu.HXRD(cls.mat.Q(1, 1, 0), cls.mat.Q(0, 0, 1))
        cls.nch = (9, 13)
        cls.ncch1 = 4
        cls.ncch2 = 6
        cls.distance = 0.5
        cls.dpix = 50e-6
        cls.hxrd.Ang2Q.init_area(
            "z+",
            "x+",
            cls.ncch1,
            cls.ncch2,
            cls.nch[0],
            cls.nch[1],
            cls.distance,
            cls.dpix,
            cls.dpix,
        )
        cls.hklsym = (0, 0, 4)
        cls.hklasym = (2, 2, 4)

    def test_qconversion_area(self):
        ang = self.hxrd.Q2Ang(self.mat.Q(self.hklsym))
        qout = self.hxrd.Ang2HKL(ang[0], ang[3], mat=self.mat, dettype="area")
        self.assertEqual(qout[0].shape, self.nch)
        for i in range(3):
            q = qout[i]
            self.assertAlmostEqual(numpy.average(q), self.hklsym[i], places=6)

    def test_qconversion_area_asym(self):
        ang = self.hxrd.Q2Ang(self.mat.Q(self.hklasym))
        qout = self.hxrd.Ang2HKL(ang[0], ang[3], mat=self.mat, dettype="area")
        self.assertEqual(qout[0].shape, self.nch)
        for i in range(3):
            q = qout[i]
            self.assertAlmostEqual(numpy.average(q), self.hklasym[i], places=6)

    def test_qconversion_area_energy(self):
        ang1 = self.hxrd.Q2Ang(self.mat.Q(self.hklsym))
        ang2 = self.hxrd.Q2Ang(self.mat.Q(self.hklsym) / 2.0)
        qout = self.hxrd.Ang2HKL(
            (ang1[0], ang2[0]),
            (ang1[3], ang2[3]),
            en=(self.hxrd.energy, 2 * self.hxrd.energy),
            mat=self.mat,
            dettype="area",
        )
        for i in range(3):
            q = qout[i]
            self.assertAlmostEqual(
                q[0, self.ncch1, self.ncch2], self.hklsym[i], places=10
            )
            self.assertAlmostEqual(
                q[1, self.ncch1, self.ncch2], self.hklsym[i], places=10
            )

    def test_qconversion_area_detpos(self):
        tt = numpy.random.rand() * 90
        dpos = self.hxrd.Ang2Q.getDetectorPos(tt, dim=2)
        dp = [p.flat for p in dpos]
        ki = (
            self.hxrd._A2QConversion.r_i
            / numpy.linalg.norm(self.hxrd._A2QConversion.r_i)
            * self.hxrd.k0
        )
        qout = self.hxrd.Ang2Q.area(0, tt)
        J1, J2 = numpy.indices(qout[0].shape)
        for j1, j2, x, y, z in zip(J1.flat, J2.flat, dp[0], dp[1], dp[2]):
            vpos = numpy.asarray((x, y, z))
            kf = vpos / numpy.linalg.norm(vpos) * self.hxrd.k0
            for i in range(3):
                self.assertAlmostEqual(
                    qout[i][j1, j2], kf[i] - ki[i], places=10
                )

    def test_qconversion_area_distance(self):
        tt = numpy.random.rand() * 140
        ddis = self.hxrd.Ang2Q.getDetectorDistance(tt, dim=2)
        J1, J2 = numpy.indices(ddis.shape)
        for j1, j2, d in zip(J1.flat, J2.flat, ddis.flat):
            dpos = (
                (j1 - self.ncch1) * self.dpix,
                self.distance,
                (j2 - self.ncch2) * self.dpix,
            )
            dis = numpy.linalg.norm(dpos)
            self.assertAlmostEqual(d, dis, places=10)


if __name__ == "__main__":
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2016-2023 Dominik Kriegner <dominik.kriegner@gmail.com>

import time

from numpy import arcsin, cos, inf, linalg, linspace, mean, pi, sqrt
from matplotlib.pylab import (
    clf,
    figure,
    legend,
    mpl,
    semilogy,
    show,
    tight_layout,
    vlines,
    xlabel,
    xlim,
    ylabel,
)

import xrayutilities as xu

mpl.rcParams["font.size"] = 16.0


en = "CuKa1"  # eV
lam = xu.en2lam(en)
resol = 2 * pi / 4998  # resolution in q; to suppress buffer oscillations
H, K, L = (0, 0, 4)
qz = linspace(4.0, 5.0, 3000)

sub = xu.simpack.Layer(xu.materials.Si, inf)
#                                                   xfrom xto nsteps thickness
buf1 = xu.simpack.GradedLayerStack(
    xu.materials.SiGe, 0.2, 0.8, 100, 10000, relaxation=1.0
)
buf2 = xu.simpack.Layer(xu.materials.SiGe(0.8), 4997.02, relaxation=1.0)
lay1 = xu.simpack.Layer(xu.materials.SiGe(0.6), 49.73, relaxation=0.0)
lay2 = xu.simpack.Layer(xu.materials.SiGe(1.0), 45.57, relaxation=0.0)
# create superlattice stack
# lattice param. are adjusted to the relaxation parameter of the layers
# Note that to create a superlattice you can use summation and multiplication
pls = xu.simpack.PseudomorphicStack001(
    "SL 5/5", sub + buf1 + buf2 + 5 * (lay1 + lay2)
)

# calculate incidence angle for dynamical diffraction models
qx = sqrt(sub.material.Q(H, K, L)[0] ** 2 + sub.material.Q(H, K, L)[1] ** 2)
ai = xu.simpack.coplanar_alphai(qx, qz, en)
resolai = abs(
    xu.simpack.coplanar_alphai(qx, mean(qz) + resol, en)
    - xu.simpack.coplanar_alphai(qx, mean(qz), en)
)

# comparison of different diffraction models
# simplest kinematical diffraction model
t0 = time.time()
mk = xu.simpack.KinematicalModel(pls, energy=en, resolution_width=resol)
Ikin = mk.simulate(qz, hkl=(0, 0, 4))
t1 = time.time()
print("%.3f sec for kinematical calculation" % (t1 - t0))

# kinematical multibeam model
t0 = time.time()
mk = xu.simpack.KinematicalMultiBeamModel(
    pls, energy=en, surface_hkl=(0, 0, 1), resolution_width=resol
)
Imult = mk.simulate(qz, hkl=(H, K, L), refraction=True)
t1 = time.time()
print("%.3f sec for kinematical multibeam calculation" % (t1 - t0))

# general 2-beam theory based dynamical diffraction model
t0 = time.time()
qGe220 = linalg.norm(xu.materials.Ge.Q(2, 2, 0))
thMono = arcsin(qGe220 * lam / (4 * pi))
md = xu.simpack.DynamicalModel(
    pls,
    energy=en,
    resolution_width=resolai,
    Cmono=cos(2 * thMono),
    polarization="both",
)
Idyn = md.simulate(ai, hkl=(H, K, L))
t1 = time.time()
print("%.3f sec for acurate dynamical calculation" % (t1 - t0))

# plot of calculated intensities
figure("XU-simpack SiGe SL")
clf()
semilogy(qz, Ikin, label="kinematical")
semilogy(qz, Imult, label="multibeam")
semilogy(xu.simpack.get_qz(qx, ai, en), Idyn, label="full dynamical")
vlines(
    [4 * 2 * pi / layer.material.a3[-1] for layer in pls[-2:]],
    1e-9,
    1,
    linestyles="dashed",
    label="kin. peak-pos",
)
legend(fontsize="small")
xlim(qz.min(), qz.max())
xlabel(r"Qz ($1/\mathrm{\AA}$)")
ylabel("Intensity (arb. u.)")
tight_layout()
show()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2009 Eugen Wintersberger <eugen.wintersberger@desy.de>
# Copyright (C) 2010-2025 Dominik Kriegner <dominik.kriegner@gmail.com>

from .algebra import solve_quartic  # noqa: F401
from .fit import (  # noqa: F401
    fit_peak2d,
    gauss_fit,
    linregress,
    multPeakFit,
    multPeakPlot,
    peak_fit,
)
from .functions import (  # noqa: F401
    Debye1,
    Gauss1d,
    Gauss1d_der_p,
    Gauss1d_der_x,
    Gauss1dArea,
    Gauss2d,
    Gauss2dArea,
    Gauss3d,
    Lorentz1d,
    Lorentz1d_der_p,
    Lorentz1d_der_x,
    Lorentz1dArea,
    Lorentz2d,
    NormGauss1d,
    NormLorentz1d,
    PseudoVoigt1d,
    PseudoVoigt1d_der_p,
    PseudoVoigt1d_der_x,
    PseudoVoigt1dArea,
    PseudoVoigt1dasym,
    PseudoVoigt1dasym2,
    PseudoVoigt2d,
    TwoGauss2d,
    heaviside,
    kill_spike,
    multPeak1d,
    multPeak2d,
    smooth,
)
from .misc import center_of_mass, derivative, fwhm_exp, gcd  # noqa: F401
from .transforms import (  # noqa: F401
    ArbRotation,
    AxisToZ,
    AxisToZ_keepXY,
    CoordinateTransform,
    Transform,
    VecAngle,
    VecCross,
    VecDot,
    VecNorm,
    VecUnit,
    XRotation,
    YRotation,
    ZRotation,
    distance,
    getSyntax,
    getVector,
    rotarb,
)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2016-2023 Dominik Kriegner <dominik.kriegner@gmail.com>

from numpy import linspace
from matplotlib.pylab import (
    clf,
    figure,
    legend,
    mpl,
    plot,
    plt,
    semilogy,
    show,
    subplot,
    tight_layout,
    twinx,
    xlabel,
    xlim,
    ylabel,
    ylim,
)
from scipy.special import erf

import xrayutilities as xu

mpl.rcParams["font.size"] = 16.0
en = "CuKa1"
qz = linspace(4.3, 4.6, 4000)

GaAs = xu.materials.GaAs
exp = xu.HXRD(GaAs.Q(1, 1, 0), GaAs.Q(0, 0, 1), en=en)
dm = xu.simpack.DarwinModelAlGaAs001(
    qz, experiment=exp, resolution_width=0.0005, I0=2e7, background=1e0
)


def period(xavg, xwell, wellratio, thick, sigmaup, sigmadown):
    """
    chemical composition profile in the superlattice
    """
    twell = wellratio * thick
    xb = (xavg * thick - twell * xwell) / (thick - twell)
    tb = (thick - twell) / 2
    print(
        "using barrier Al-content, well thickness, barrier thickness/2: "
        f"{xb:.3f} {twell:.1f} {tb:.1f}"
    )
    return (
        lambda z: xb
        + (xwell - xb)
        * (erf((z - tb) / sigmaup) + erf(-(z - (tb + twell)) / sigmadown))
        / 2.0
    )


sideal = [
    {"t": 3500000, "x": 0, "r": 1},  # 350um substrate
    {"t": 10000, "x": 0.5, "r": 1},  # 1um buffer (relaxed)
    # 5 period superlattice with interdiffusion
    (
        5,
        [
            {"t": 75, "x": 0.9, "r": 0},
            {"t": 150, "x": 0.0, "r": 0},
            {"t": 75, "x": 0.9, "r": 0},
        ],
    ),
    {"t": 100, "x": 0, "r": 0},
]  # 10nm cap
sample = [
    {"t": 3500000, "x": 0, "r": 1},  # 350um substrate
    {"t": 10000, "x": 0.5, "r": 1},  # 1um buffer (relaxed)
    # 5 period superlattice with interdiffusion
    (5, [{"t": 300, "x": period(0.45, 0, 0.5, 300, 20, 20), "r": 0.0}]),
    {"t": 100, "x": 0, "r": 0},
]  # 10nm cap

# perform Darwin-theory based simulation
mlideal = dm.make_monolayers(sideal)
Iideal = dm.simulate(mlideal)
ml = dm.make_monolayers(sample)
Isim = dm.simulate(ml)

figure("XU-simpack AlGaAs (Darwin)", figsize=(10, 5))
clf()

subplot(121)
semilogy(qz, Iideal, "-m", lw=2, label="ideal")
semilogy(qz, Isim, "-r", lw=2, label="SL5")
ylim(0.5 * dm.background, dm.I0)
xlim(qz.min(), qz.max())
plt.locator_params(axis="x", nbins=5)
xlabel(r"Qz ($1/\mathrm{\AA}$)")
ylabel("Intensity (arb. u.)")
legend(fontsize="small")

subplot(122)
z, xAl = dm.prop_profile(mlideal, "x")
plot(z / 10, xAl, "-m")
z, xAl = dm.prop_profile(ml, "x")
plot(z / 10, xAl, "-r")
xlabel("depth z (nm)")
ylabel("Al-content", color="m")
ylim(-0.05, 1.05)
twinx()
z, ai = dm.prop_profile(ml, "ai")
plot(z / 10, ai, "-b")
xlim(-1200, 3)
ylim(5.64, 5.68)
plt.locator_params(axis="x", nbins=5)
ylabel(r"a-inplane ($\mathrm{\AA}$)", color="b")
tight_layout()
show()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2014-2020 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
import unittest

import xrayutilities as xu

xu.config.VERBOSITY = 0  # make no output during test
testfile = "alignment.log.gz"
datadir = os.path.join(os.path.dirname(__file__), "data")
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(
    not os.path.isfile(fullfilename),
    "additional test data needed (http://xrayutilities.sf.io)",
)
class TestIO_SPEC_RA_Log(unittest.TestCase):
    peaks = ["asymaz1", "symaz1"]
    niterations = 639

    @classmethod
    def setUpClass(cls):
        cls.logfile = xu.io.RA_Alignment(fullfilename)

    def test_peaknames(self):
        self.assertEqual(self.peaks, self.logfile.peaks)

    def test_iterations(self):
        self.assertEqual(
            self.niterations, sum([sum(i) for i in self.logfile.iterations])
        )


if __name__ == "__main__":
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2009 Eugen Wintersberger <eugen.wintersberger@desy.de>
# Copyright (C) 2009-2025 Dominik Kriegner <dominik.kriegner@gmail.com>

"""xrayutilities assisting with x-ray diffraction experiments.

It helps with planning experiments as well as analyzing the data.

Authors:
 Dominik Kriegner <dominik.kriegner@gmail.com> and
 Eugen Wintersberger <eugen.wintersberger@desy.de>
"""

from importlib.metadata import version

# load configuration
from . import analysis, config, io, materials, math, simpack  # noqa: F401
from .experiment import (  # noqa: F401
    GID,
    GISAXS,
    HXRD,
    Experiment,
    FourC,
    NonCOP,
    PowderExperiment,
    QConversion,
)
from .gridder import FuzzyGridder1D, Gridder1D, npyGridder1D  # noqa: F401
from .gridder2d import FuzzyGridder2D, Gridder2D, Gridder2DList  # noqa: F401
from .gridder3d import FuzzyGridder3D, Gridder3D  # noqa: F401
from .normalize import (  # noqa: F401
    IntensityNormalizer,
    blockAverage1D,
    blockAverage2D,
    blockAverageCCD,
    blockAveragePSD,
)
from .q2ang_fit import Q2AngFit  # noqa: F401
from .utilities import (  # noqa: F401
    en2lam,
    energy,
    frac2str,
    lam2en,
    makeNaturalName,
    maplog,
    wavelength,
)

# load package version
__version__ = version("xrayutilities")



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2025 Dominik Kriegner <dominik.kriegner@gmail.com>

import time
import unittest

import numpy as np
import xrayutilities as xu
from scipy.stats import uniform

# get maximum number of threads
# if this is -1 no openmp is available and we skip the test class
max_threads = xu.cxrayutilities.get_max_openmp_threads()


@unittest.skipIf(max_threads == -1, "extension was not compiled with OpenMP")
class TestParallelPerformance(unittest.TestCase):
    def setUp(self):
        """Set up the HXRD object and generate random angles."""
        mat = xu.materials.Si  # Example material
        self.hxrd = xu.HXRD(mat.Q(1, 1, 0), mat.Q(0, 0, 1))
        self.num_angles = 10000000  # Number of random angles to test
        self.tt_angles = uniform.rvs(size=self.num_angles, loc=0, scale=80)
        self.pp_angles = uniform.rvs(size=self.num_angles, loc=0, scale=140)

    def test_parallel_performance(self):
        """Test the performance scaling of parallel processing.

        Tests angle to Q-space conversion performance with varying numbers of
        threads. Calculates speedup and efficiency metrics for different thread
        counts. Checks that parallel processing achieves reasonable efficiency
        thresholds.
        """
        if max_threads < 2:
            self.skipTest("OpenMP available, but not enough allowed threads")
            return
        times = {}
        n_threads_tested = list(range(1, min(13, max_threads + 1)))
        for nthreads in n_threads_tested:
            xu.config.NTHREADS = nthreads
            start_time = time.time()
            qx, qy, qz = self.hxrd.Ang2Q(self.pp_angles, self.tt_angles)
            end_time = time.time()
            times[nthreads] = end_time - start_time
            print(f"Nthreads: {nthreads}, Time: {times[nthreads]:.4f} seconds")

        # Calculate Efficiency
        efficiencies = {}
        time_1 = times[1]  # Time for 1 thread
        for nthreads in n_threads_tested:
            speedup = time_1 / times[nthreads]
            efficiency = speedup / nthreads
            efficiencies[nthreads] = efficiency
            print(
                f"Nthreads: {nthreads}, Speedup: {speedup:.2f}, "
                f"Efficiency: {efficiency:.2f}"
            )

        # Average Efficiency (excluding 1 thread, as efficiency=1.0)
        avg_efficiency = np.mean(list(efficiencies.values())[1:])
        print(f"Average Efficiency (excluding 1 thread): {avg_efficiency:.2f}")

        # Assertions based on the average efficiency
        self.assertGreater(
            avg_efficiency, 0.25, "Average efficiency should be reasonable"
        )
        self.assertGreater(
            efficiencies[2], 0.5, "Efficiency at 2 threads should be high."
        )


if __name__ == "__main__":
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2012 Dominik Kriegner <dominik.kriegner@gmail.com>

from math import sqrt

import numpy

import xrayutilities as xu

# define elements
In = xu.materials.elements.In
P = xu.materials.elements.P

# define materials
InP = xu.materials.InP
InPWZ = xu.materials.InPWZ

# approximate version of other hexagonal polytypes
ainp = InP.a
InP4H = xu.materials.Crystal(
    "InP(4H)",
    xu.materials.SGLattice(
        186,
        ainp / sqrt(2),
        2 * sqrt(4 / 3.0) * ainp,
        atoms=[In, In, P, P],
        pos=[("2a", 0), ("2b", 1 / 4.0), ("2a", 3 / 16.0), ("2b", 7 / 16.0)],
    ),
)

for energy in [8041]:  # eV
    lam = xu.en2lam(energy)  # e in eV -> lam in angstrom
    print("         %d eV = %8.4f A" % (energy, lam))
    print(
        "------------------------------------------------------------------"
        "-----------------"
    )
    print(
        "material |         peak    |   omega  |  2theta  |    phi   |   "
        "tt-om  |     |F|   "
    )
    print(
        "------------------------------------------------------------------"
        "-----------------"
    )

    exp111 = xu.HXRD(InP.Q(1, 1, -2), InP.Q(1, 1, 1), en=energy)
    exphex = xu.HXRD(InPWZ.Q(1, -1, 0), InPWZ.Q(0, 0, 1), en=energy)

    # InP ZB reflections
    reflections = [
        [1, 1, 1],
        [2, 2, 2],
        [3, 3, 3],
        [3, 3, 1],
        [2, 2, 4],
        [1, 1, 5],
    ]
    mat = InP
    for hkl in reflections:
        qvec = mat.Q(hkl)
        [om, chi, phi, tt] = exp111.Q2Ang(qvec, trans=True)
        F = mat.StructureFactor(qvec, exp111._en)
        F /= mat.lattice.UnitCellVolume()
        print(
            "%8s | %15s | %8.4f | %8.4f | %8.4f | %8.4f | %8.2f "
            % (
                mat.name,
                " ".join(map(str, numpy.round(hkl, 2))),
                om,
                tt,
                phi,
                tt - om,
                numpy.abs(F),
            )
        )

    print(
        "------------------------------------------------------------------"
        "-----------------"
    )
    # InP WZ reflections
    reflections = [[0, 0, 2], [0, 0, 4], [1, -1, 4], [1, -1, 5], [1, -1, 6]]
    mat = InPWZ
    for hkl in reflections:
        qvec = mat.Q(hkl)
        [om, chi, phi, tt] = exphex.Q2Ang(qvec, trans=True)
        F = mat.StructureFactor(qvec, exphex._en)
        F /= mat.lattice.UnitCellVolume()
        print(
            "%8s | %15s | %8.4f | %8.4f | %8.4f | %8.4f | %8.2f "
            % (
                mat.name,
                " ".join(map(str, numpy.round(hkl, 2))),
                om,
                tt,
                phi,
                tt - om,
                numpy.abs(F),
            )
        )

    print(
        "------------------------------------------------------------------"
        "-----------------"
    )
    # InP 4H
    reflections = [[0, 0, 4], [0, 0, 8], [1, -1, 9], [1, -1, 10], [1, -1, 11]]
    mat = InP4H
    for hkl in reflections:
        qvec = mat.Q(hkl)
        [om, chi, phi, tt] = exphex.Q2Ang(qvec, trans=True)
        F = mat.StructureFactor(qvec, exphex._en)
        F /= mat.lattice.UnitCellVolume()
        print(
            "%8s | %15s | %8.4f | %8.4f | %8.4f | %8.4f | %8.2f "
            % (
                mat.name,
                " ".join(map(str, numpy.round(hkl, 2))),
                om,
                tt,
                phi,
                tt - om,
                numpy.abs(F),
            )
        )

    print(
        "------------------------------------------------------------------"
        "-----------------"
    )



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2009 Eugen Wintersberger <eugen.wintersberger@desy.de>
# Copyright (c) 2010-2020, 2023 Dominik Kriegner <dominik.kriegner@gmail.com>
# Copyright (C) 2022 Vinícius Frehse <vinifrehse@gmail.com>

from . import elements  # noqa: F401
from .atom import Atom  # noqa: F401
from .cif import CIFFile, cifexport  # noqa: F401
from .database import (  # noqa: F401
    DataBase,
    add_f0_from_intertab,
    add_f0_from_xop,
    add_f1f2_from_ascii_file,
    add_f1f2_from_henkedb,
    add_f1f2_from_kissel,
    add_mass_from_NIST,
    init_material_db,
)
from .material import (  # noqa: F401
    Alloy,
    Amorphous,
    Crystal,
    CubicAlloy,
    CubicElasticTensor,
    HexagonalElasticTensor,
    Material,
    MonoclinicElasticTensor,
    PseudomorphicMaterial,
    TrigonalElasticTensor,
    WZTensorFromCub,
)
from .plot import show_reciprocal_space_plane  # noqa: F401
from .predefined_materials import *  # noqa: F403
from .spacegrouplattice import SGLattice, SymOp  # noqa: F401



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2019-2023 Dominik Kriegner <dominik.kriegner@gmail.com>

from numpy import arcsin, cos, inf
from matplotlib.pylab import (
    figure,
    legend,
    plot,
    semilogy,
    show,
    tight_layout,
    xlabel,
    xlim,
    ylabel,
    ylim,
)

import xrayutilities as xu


# global parameters
wavelength = xu.wavelength("CuKa1")
offset = -0.035  # angular offset of the zero position of the data

# set up LayerStack for simulation: InAs(001)/(In,Mn)As(~25 nm)
InAs = xu.materials.InAs
InAs.lattice.a = 6.057
lInAs = xu.simpack.Layer(InAs, inf)
InMnAs = xu.materials.Crystal(
    "InMnAs",
    xu.materials.SGLattice(
        216,
        6.050,
        atoms=("In", "Mn", "As"),
        pos=("4a", "4a", "4c"),
        occ=(0.88, 0.12, 1),
    ),
    cij=InAs.cij,
)
lInMnAs = xu.simpack.Layer(InMnAs, 254, relaxation=0)
pstack = xu.simpack.PseudomorphicStack001("list", lInAs, lInMnAs)

# set up simulation object
thetaMono = arcsin(wavelength / (2 * xu.materials.Ge.planeDistance(2, 2, 0)))
Cmono = cos(2 * thetaMono)
dyn = xu.simpack.DynamicalModel(
    pstack,
    I0=1.5e9,
    background=0,
    resolution_width=2e-3,
    polarization="both",
    Cmono=Cmono,
)
fitmdyn = xu.simpack.FitModel(dyn)
fitmdyn.set_param_hint("InMnAs_c", vary=True, min=6.02, max=6.06)
fitmdyn.set_param_hint("InAs_a", vary=True)
fitmdyn.set_param_hint("InMnAs_a", expr="InAs_a")
fitmdyn.set_param_hint("resolution_width", vary=True)
params = fitmdyn.make_params()

# plot experimental data
f = figure(figsize=(7, 5))
d = xu.io.RASFile("inas_layer_radial_002_004.ras.bz2", path="data")
scan = d.scans[-1]
tt = scan.data[scan.scan_axis] - offset
semilogy(tt, scan.data["int"], "o-", ms=3, label="data")

# perform fit and plot the result
fitmdyn.lmodel.set_hkl((0, 0, 4))
ai = (d.scans[-1].data[d.scan.scan_axis] - offset) / 2
fitr = fitmdyn.fit(d.scans[-1].data["int"], params, ai)
print(fitr.fit_report())

simdyn = fitmdyn.eval(fitr.params, x=tt / 2)
plot(tt, simdyn, label="sim")

# label the plot
ylabel("Intensity (counts)")
xlabel("2Theta (deg)")
legend()
ylim(2e1, 1e9)
xlim(57, 65)
tight_layout()
show()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2017-2021 Dominik Kriegner <dominik.kriegner@gmail.com>

import os
import unittest
from multiprocessing import freeze_support

import numpy
import xrayutilities as xu

try:
    import lmfit
except ImportError:
    lmfit = None

testfile = "LaB6_d500_si_psd.xye.bz2"
datadir = os.path.join(os.path.dirname(__file__), "data")
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(
    not os.path.isfile(fullfilename) or lmfit is None,
    "additional test data (see http://xrayutilities.sf.io) and "
    "the lmfit Python package are needed",
)
class Test_PowderModel(unittest.TestCase):
    chi2max = 1.5
    # define powder material
    La = xu.materials.elements.La
    B = xu.materials.elements.B
    LaB6 = xu.materials.Crystal(
        "LaB6",
        xu.materials.SGLattice(
            221,
            4.15692,
            atoms=[La, B],
            pos=["1a", ("6f", 0.19750)],
            b=[0.05, 0.15],
        ),
    )
    LaB6_powder = xu.simpack.Powder(
        LaB6,
        1,
        crystallite_size_gauss=1e6,
        crystallite_size_lor=0.5e-6,
        strain_gauss=0,
        strain_lor=0,
    )
    # machine settings
    settings = {
        "classoptions": {"oversampling": 10},
        "global": {
            "diffractometer_radius": 0.337,
            "equatorial_divergence_deg": 0.40,
        },
        "tube_tails": {
            "tail_left": -0.001,
            "main_width": 0.00015,
            "tail_right": 0.001,
            "tail_intens": 0.0015,
        },
        "axial": {
            "angI_deg": 2.0,
            "angD_deg": 2.0,
            "slit_length_target": 0.008,
            "n_integral_points": 21,
            "length_sample": 0.015,
            "slit_length_source": 0.008001,
        },
        "si_psd": {"si_psd_window_bounds": (0, 32e-3)},
        "absorption": {
            "sample_thickness": 500e-6,
            "absorption_coefficient": 3e4,
        },
        "displacement": {
            "specimen_displacement": -3.8e-5,
            "zero_error_deg": 0.0,
        },
        "emission": {"emiss_intensities": (1.0, 0.45)},
    }
    # define background
    btt, bint = numpy.asarray(
        [
            (15.158, 1136.452),
            (17.886, 841.925),
            (22.906, 645.784),
            (26.556, 551.663),
            (34.554, 401.219),
            (45.764, 260.595),
            (58.365, 171.993),
            (81.950, 112.838),
            (92.370, 101.276),
            (106.441, 102.486),
            (126.624, 112.838),
            (139.096, 132.063),
            (146.240, 136.500),
            (152.022, 157.204),
        ]
    ).T

    def setUp(self):
        with xu.io.xu_open(fullfilename) as fid:
            self.tt, self.det, self.sig = numpy.loadtxt(fid, unpack=True)
        self.mask = numpy.logical_and(self.tt > 18, self.tt < 148)
        self.pm = xu.simpack.PowderModel(
            self.LaB6_powder, I0=1.10e6, fpsettings=self.settings
        )
        self.pm.set_background("spline", x=self.btt, y=self.bint)

    def test_Calculation(self):
        x = numpy.arange(
            10,
            140 + numpy.random.rand() * 10,
            0.0075 + numpy.random.rand() * 0.005,
        )
        sim = self.pm.simulate(x)
        self.pm.close()
        self.assertEqual(len(sim), len(x))
        self.assertTrue(numpy.all(sim >= 0))

    def test_multiprocessing(self):
        x = numpy.arange(
            10,
            140 + numpy.random.rand() * 10,
            0.0075 + numpy.random.rand() * 0.005,
        )
        st_sim = self.pm.simulate(x, mode="local")
        mt_sim = self.pm.simulate(x, mode="multi")
        self.pm.close()
        self.assertAlmostEqual(numpy.sum(numpy.abs(st_sim - mt_sim)), 0.0)

    def test_fitting(self):
        # first fit run
        p = self.pm.create_fitparameters()
        for pn, limit in (
            ("primary_beam_intensity", (None, None)),
            ("displacement_specimen_displacement", (-1e-4, 1e-4)),
            ("displacement_zero_error_deg", (-0.01, 0.01)),
        ):
            p[pn].set(vary=True, min=limit[0], max=limit[1])

        fitres1 = self.pm.fit(
            p,
            self.tt[self.mask],
            self.det[self.mask],
            std=self.sig[self.mask],
            maxfev=50,
        )

        # second fit run to optimize absorption
        p = fitres1.params
        for pn, limit in (
            ("primary_beam_intensity", (None, None)),
            ("displacement_specimen_displacement", (-1e-4, 1e-4)),
            ("absorption_absorption_coefficient", (1e4, 10e4)),
        ):
            p[pn].set(vary=True, min=limit[0], max=limit[1])

        fitres2 = self.pm.fit(
            p, self.tt[self.mask], self.det[self.mask], std=self.sig[self.mask]
        )
        fitsim = self.pm.simulate(self.tt[self.mask])
        M, Rp, Rwp, Rwpexp, chi2 = xu.simpack.Rietveld_error_metrics(
            self.det[self.mask],
            fitsim,
            std=self.sig[self.mask],
            Nvar=fitres2.nvarys,
            disp=False,
        )
        self.pm.close()

        self.assertTrue(chi2 < self.chi2max)


if __name__ == "__main__":
    freeze_support()  # required for MS Windows
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2018-2023 Dominik Kriegner <dominik.kriegner@gmail.com>

import time

from numpy import arange, inf
from matplotlib.pylab import (
    figure,
    legend,
    semilogy,
    tight_layout,
    xlabel,
    ylabel,
)

import xrayutilities as xu

sub = xu.simpack.Layer(xu.materials.Si, inf, roughness=1, lat_correl=100)
lay1 = xu.simpack.Layer(xu.materials.Si, 200, roughness=1, lat_correl=200)
lay2 = xu.simpack.Layer(xu.materials.Ge, 70, roughness=3, lat_correl=50)

ls = xu.simpack.LayerStack("SL 5", sub + 5 * (lay2 + lay1))

alphai = arange(0.17, 2, 0.001)

print("calculate method=1, H=1, vert=0")
start = time.time()
m = xu.simpack.DiffuseReflectivityModel(
    ls,
    sample_width=10,
    beam_width=1,
    energy="CuKa1",
    vert_correl=1000,
    vert_nu=0,
    H=1,
    method=1,
    vert_int=0,
)
d1 = m.simulate(alphai)
print(f"elapsed time: {time.time() - start:.4f}")


print("calculate method=2, H=1, vert=0")
start = time.time()
m = xu.simpack.DiffuseReflectivityModel(
    ls,
    sample_width=10,
    beam_width=1,
    energy="CuKa1",
    vert_correl=1000,
    vert_nu=0,
    H=1,
    method=2,
    vert_int=0,
)
d2 = m.simulate(alphai)
print("elapsed time: %.4f" % (time.time() - start))

figure()
semilogy(alphai, d1, label="method=1")
semilogy(alphai, d2, label="method=2")

legend()
xlabel("incidence angle (deg)")
ylabel("intensity (arb. u.)")
tight_layout()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2019-2025 Dominik Kriegner <dominik.kriegner@gmail.com>

import configparser
import os
import subprocess
import sys
import tempfile
import unittest
from pathlib import Path

scriptdir = Path(__file__).parent.parent / "examples"
scriptfiles = [
    "simpack_powdermodel.py",
    "simpack_xrd_AlGaAs.py",
    "simpack_xrd_Darwin_AlGaAs.py",
    "simpack_xrd_dyn_AlGaAs.py",
    "simpack_xrd_InAs_fitting.py",
    "simpack_xrd_SiGe111.py",
    "simpack_xrd_SiGe_asymmmetric.py",
    "simpack_xrd_SiGe.py",
    "simpack_xrd_SiGe_superlattice.py",
    "simpack_xrr_diffuse.py",
    "simpack_xrr_matrixmethod.py",
    "simpack_xrr_SiO2_Ru_CoFe_IrMn_Al2O3.py",
    "xrayutilities_angular2hkl_conversion.py",
    # "xrayutilities_ccd_parameter.py",  # data file not included
    "xrayutilities_components_of_the_structure_factor.py",
    "xrayutilities_define_material.py",
    "xrayutilities_energy_dependent_structure_factor.py",
    # "xrayutilities_example_plot_3D_ESRF_ID01.py",  # data file not included
    "xrayutilities_experiment_angle_calculation.py",
    "xrayutilities_experiment_kappa.py",
    "xrayutilities_experiment_Powder_example_Iron.py",
    # "xrayutilities_export_data2vtk.py",  # needs vtk + data
    "xrayutilities_fuzzygridding.py",
    "xrayutilities_hotpixelkill_variant.py",
    "xrayutilities_id01_functions.py",
    "xrayutilities_io_cif_parser_bi2te3.py",
    "xrayutilities_io_cif_parser.py",
    # "xrayutilities_io_pdcif_plot.py",  # data file not included
    # "xrayutilities_kmap_example_ESRF.py",  # data file not included
    "xrayutilities_materials_Alloy_contentcalc.py",
    "xrayutilities_math_fitting.py",
    "xrayutilities_orientation_matrix.py",
    "xrayutilities_peak_angles_beamtime.py",
    # "xrayutilities_polefigure.py",  # basemap needed
    "xrayutilities_q2ang_general.py",
    "xrayutilities_read_panalytical.py",
    # "xrayutilities_read_seifert.py",  # data file not included
    "xrayutilities_read_spec.py",
    "xrayutilities_reflection_strength.py",
    "xrayutilities_show_reciprocal_space_plane.py",
]

cleanup_files = [
    "xrrfit.dat",
]


class TestExampleScripts(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """Create temporary config settings.

        Limit number of used threads during the test run of the examples.
        """
        cls.config_file = scriptdir / "xrayutilities.conf"
        cls.config = configparser.ConfigParser()
        cls.config["xrayutilities"] = {"nthreads": "2"}

        with open(cls.config_file, "w") as f:
            cls.config.write(f)

    def test_examples(self):
        """Testrun example scripts."""
        for sf in scriptfiles:
            print(f"starting script {sf}")
            with self.subTest(script=sf):
                with tempfile.TemporaryFile(mode="w") as fid:
                    env = os.environ.copy()
                    env["MPLBACKEND"] = "agg"
                    cmd = [sys.executable, sf]
                    subprocess.run(
                        cmd, env=env, cwd=scriptdir, stdout=fid, check=True
                    )

    @classmethod
    def tearDownClass(cls):
        """Clean up any files created during tests."""
        try:
            os.remove(cls.config_file)
        except FileNotFoundError:
            print(
                f"Warning: Config file {cls.config_file} not found during "
                "tearDownClass."
            )
        for f in cleanup_files:
            try:
                Path(scriptdir, f).unlink(missing_ok=True)
            except OSError:
                pass


if __name__ == "__main__":
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2016-2023 Dominik Kriegner <dominik.kriegner@gmail.com>

from numpy import inf, linspace, mean
from matplotlib.pylab import (
    clf,
    figure,
    legend,
    mpl,
    semilogy,
    show,
    tight_layout,
    vlines,
    xlabel,
    xlim,
    ylabel,
)

import xrayutilities as xu

mpl.rcParams["font.size"] = 16.0


en = 8500  # eV
resol = 0.0004  # resolution in q
H, K, L = (1, 1, 1)
qz = linspace(1.8, 2.2, 5000)
Si = xu.materials.Si
hxrd = xu.HXRD(Si.Q(1, 1, -2), Si.Q(1, 1, 1), en=en)

sub = xu.simpack.Layer(Si, inf)
lay = xu.simpack.Layer(xu.materials.SiGe(0.6), 150, relaxation=0.5)
pls = xu.simpack.PseudomorphicStack111("pseudo", sub, lay)

# calculate incidence angle for dynamical diffraction models
qx = hxrd.Transform(Si.Q(H, K, L))[1]
ai = xu.simpack.coplanar_alphai(qx, qz, en)
resolai = abs(
    xu.simpack.coplanar_alphai(qx, mean(qz) + resol, en)
    - xu.simpack.coplanar_alphai(qx, mean(qz), en)
)

# comparison of different diffraction models
# simplest kinematical diffraction model
mk = xu.simpack.KinematicalModel(pls, experiment=hxrd, resolution_width=resol)
Ikin = mk.simulate(qz, hkl=(H, K, L), refraction=True)

# simplified dynamical diffraction model
mds = xu.simpack.SimpleDynamicalCoplanarModel(
    pls, experiment=hxrd, resolution_width=resolai
)
Idynsub = mds.simulate(ai, hkl=(H, K, L), idxref=0)
Idynlay = mds.simulate(ai, hkl=(H, K, L), idxref=1)

# general 2-beam theory based dynamical diffraction model
md = xu.simpack.DynamicalModel(pls, experiment=hxrd, resolution_width=resolai)
Idyn = md.simulate(ai, hkl=(H, K, L))

# plot of calculated intensities
figure("XU-simpack SiGe(111)")
clf()
semilogy(qz, Ikin, label="kinematical")
semilogy(xu.simpack.get_qz(qx, ai, en), Idynsub, label="simpl. dynamical(S)")
semilogy(xu.simpack.get_qz(qx, ai, en), Idynlay, label="simpl. dynamical(L)")
semilogy(xu.simpack.get_qz(qx, ai, en), Idyn, label="full dynamical")
vlines(
    [xu.math.VecNorm(lay.material.Q(H, K, L)) for lay in pls],
    1e-9,
    1,
    linestyles="dashed",
)
legend(fontsize="small")
xlim(qz.min(), qz.max())
xlabel(r"Qz ($1/\mathrm{\AA}$)")
ylabel("Intensity (arb. u.)")
tight_layout()
show()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2016-2022 Dominik Kriegner <dominik.kriegner@gmail.com>

import os

import lmfit
import matplotlib.pylab as pylab
import numpy

import xrayutilities as xu

# load experimental data
ai, edata, eps = numpy.loadtxt(
    os.path.join("data", "xrr_data.txt"), unpack=True
)
ai /= 2.0

# define layers
# SiO2 / Ru(5) / CoFe(3) / IrMn(3) / AlOx(10)
lSiO2 = xu.simpack.Layer(xu.materials.SiO2, numpy.inf, roughness=2.5)
lRu = xu.simpack.Layer(xu.materials.Ru, 47, roughness=2.8)
rho_cf = 0.5 * 8900 + 0.5 * 7874
mat_cf = xu.materials.Amorphous("CoFe", rho_cf)
lCoFe = xu.simpack.Layer(mat_cf, 27, roughness=4.6)
lIrMn = xu.simpack.Layer(xu.materials.Ir20Mn80, 21, roughness=3.0)
lAl2O3 = xu.simpack.Layer(xu.materials.Al2O3, 100, roughness=5.5)

# create model
m = xu.simpack.SpecularReflectivityModel(
    lSiO2,
    lRu,
    lCoFe,
    lIrMn,
    lAl2O3,
    energy="CuKa1",
    resolution_width=0.02,
    sample_width=6,
    beam_width=0.25,
    background=81,
    I0=6.35e9,
)

# embed model in fit code
fitm = xu.simpack.FitModel(m, plot=True, verbose=True)

# set some parameter limitations
fitm.set_param_hint("SiO2_density", vary=False)
fitm.set_param_hint(
    "Al2O3_density",
    min=0.8 * xu.materials.Al2O3.density,
    max=1.2 * xu.materials.Al2O3.density,
)
p = fitm.make_params()
fitm.set_fit_limits(xmin=0.05, xmax=8.0)

# perform the fit
res = fitm.fit(edata, p, ai, weights=1 / eps)
lmfit.report_fit(res, min_correl=0.5)

m.densityprofile(500, plot=True)
pylab.show()

# export the fit result for the full data range (Note that only data between
# xmin and xmax were actually optimized)
numpy.savetxt(
    "xrrfit.dat",
    numpy.vstack((ai, res.eval(res.params, x=ai))).T,
    header="incidence angle (deg), fitted intensity (arb. u.)",
)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2012, 2023 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
from itertools import permutations

import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy
from mpl_toolkits.basemap import Basemap

import xrayutilities as xu

datadir = "data"

eps = 0.01
# plot settings
mpl.rcParams["font.size"] = 18.0
mpl.rcParams["axes.labelsize"] = "large"
mpl.rcParams["legend.fancybox"] = True
mpl.rcParams["legend.handletextpad"] = 0.5
mpl.rcParams["legend.fontsize"] = "medium"
mpl.rcParams["figure.subplot.bottom"] = 0.13
mpl.rcParams["figure.subplot.top"] = 0.93
mpl.rcParams["figure.subplot.left"] = 0.14
mpl.rcParams["figure.subplot.right"] = 0.915
mpl.rcParams["savefig.dpi"] = 300
mpl.rcParams["axes.grid"] = False

# substrate
Ge = xu.materials.Ge

hkls = (
    list(permutations([1, 1, 3]))
    + list(permutations([-1, 1, 3]))
    + list(permutations([-1, -1, 3]))
    + list(permutations([1, 1, -3]))
    + list(permutations([-1, 1, -3]))
    + list(permutations([-1, -1, -3]))
)

tup = []
label = []
for ind in hkls:
    tup.append(Ge.Q(*ind))
    lab = r"$("
    for index in ind:
        lab += "%s" % (r"\bar" if index < 0 else "") + str(abs(index))
    lab += ")$"
    label.append(lab)

df = xu.io.XRDMLFile(os.path.join(datadir, "polefig_Ge113.xrdml.bz2"))
s = df.scan

chi = 90 - s["Psi"]
phi = s["Phi"]
INT = s["detector"]

# create 2D arrays for the angles
CHI = chi[:, numpy.newaxis] * numpy.ones(INT.shape)
PHI = phi

INT = xu.maplog(INT, 6, 0)

fig = plt.figure()
plt.clf()
# plt.title("(113) pole figure")
m = Basemap(
    boundinglat=-1.0, lon_0=180.0, resolution=None, projection="npstere"
)
X, Y = m(PHI, CHI)
ax = plt.subplot(111)
ax.set_frame_on(False)
CS = m.contourf(X, Y, INT, 50)
m.drawparallels(
    numpy.arange(0, 91, 10), labels=[1, 1, 1, 1], color="gray", dashes=[2, 2]
)
m.drawmeridians(
    numpy.arange(0, 360, 60),
    labels=[1, 1, 1, 1],
    color="gray",
    labelstyle="+/-",
    dashes=[2, 2],
)

dphi = 62
dchi = -32
inpdir = xu.math.rotarb(Ge.Q(1, -1, 0), Ge.Q(0, 0, 1), dphi)
ndir = xu.math.rotarb(Ge.Q(0, 0, 1), inpdir, dchi)

# plot normaldir
# calculate spherical coordinate angles
[chi, phi] = xu.analysis.getangles(Ge.Q(0, 0, 1), ndir, inpdir)
chi = 90 - chi
phi = phi - dphi
# if direction is visible plot in polefig
if chi >= -eps:
    x, y = m(phi, chi)
    m.plot(
        numpy.array([x]),
        numpy.array([y]),
        ls="None",
        marker="s",
        color="k",
        ms=12.0,
    )

# plot Ge {113} Bragg peaks
for i in range(len(tup)):
    dir = tup[i]
    # calculate spherical coordinate angles
    dir = xu.math.rotarb(dir, Ge.Q(0, 0, 1), 27)
    [chi, phi] = xu.analysis.getangles(dir, ndir, inpdir)
    chi = 90 - chi
    phi = phi - dphi
    # if direction is visible plot in polefig
    if chi >= -eps:
        x, y = m(phi, chi)
        m.plot(
            numpy.array([x]),
            numpy.array([y]),
            ls="None",
            marker="o",
            color="k",
            ms=8.0,
            mfc="None",
            mew=2.0,
        )
        plt.text(x + 400000, y, label[i], ha="left", va="bottom")



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2014 Raphael Grifone <raphael.grifone@esrf.fr>
# Copyright (c) 2014-2023 Dominik Kriegner <dominik.kriegner@gmail.com>

import numpy
import xrayutilities_id01_functions as id01
from matplotlib.pylab import (
    axis,
    figure,
    mpl,
    pcolormesh,
    subplot,
    tight_layout,
    tricontourf,
    xlabel,
    ylabel,
)

import xrayutilities as xu
from xrayutilities_id01_functions import datadir, deadpixelkill

# 3S+2D goniometer (ID01 goniometer, sample mu, eta, phi detector
# nu, del, mpxy, mpxz
# convention for coordinate system: x downstream; z upwards; y to the
# "outside" (righthanded)
# QConversion will set up the goniometer geometry. So the first argument
# describes the sample rotations, the second the detector rotations and the
# third the primary beam direction.
# For this consider the following right handed coordinate system (feel free to
# use your conventions):
# x: downstream (direction of primary beam)
# y: out of the ring
# z: upwards
# The outer most sample rotation (so the one mounted on the floor) is one
# which turns left-handed (-) around the z-direction -> z- (mu)
# The second sample rotation ('eta') is lefthanded (-) around y -> y-
qconv = xu.experiment.QConversion(
    ["z-", "y-", "z-"], ["z-", "y-", "ty", "tz"], [1, 0, 0]
)
hxrd = xu.HXRD([1, 1, 0], [0, 0, 1], qconv=qconv, sampleor="z+")
hxrd._A2QConversion.init_area(
    "z-",
    "y+",
    cch1=333.94,
    cch2=235.62,
    Nch1=516,
    Nch2=516,
    pwidth1=5.5000e-02,
    pwidth2=5.5000e-02,
    distance=0.53588 * 1000,
    detrot=-1.495,
    tiltazimuth=155.0,
    tilt=0.745,
    Nav=(2, 2),
)

#############################################################
# load spec-file data
fnames = ("KMAP_2017_fast_00001.spec",)

scannrs = []
for fn in fnames:
    sf = xu.io.SPECFile(fn, path=datadir)
    scannrs.append(list(range(len(sf))))

nx, ny = 150, 150

# specfile, scannumbers, nx,ny, motornames, optional column names (ID01
# values are default)
fss = xu.io.FastScanSeries(
    fnames,
    scannrs,
    nx,
    ny,
    "mu",
    "eta",
    "phi",
    "nu",
    "del",
    "mpxy",
    "mpxz",
    xmotor="adcY",
    ymotor="adcX",
    ccdnr="imgnr",
    path=datadir,
)

#############################################################
# 3D RSM from summing over X,Y
# now all EDF files are parsed, this will take a while and some memory
qconv.energy = id01.getmono_energy(sf[0])
xu.config.VERBOSITY = 2
g3d = fss.get_average_RSM(
    81,
    82,
    83,
    qconv,
    datadir=id01.datadir,
    replacedir=id01.repl_n,
    roi=None,
    nav=(4, 4),
    filterfunc=deadpixelkill,
)
xu.config.VERBOSITY = 1
numpy.savez_compressed(
    "RSM3D.npz", qx=g3d.xaxis, qy=g3d.yaxis, qz=g3d.zaxis, data=g3d.data
)
qx = g3d.xaxis
qy = g3d.yaxis
qz = g3d.zaxis
data = g3d.data

figure()
subplot(221)
pcolormesh(qx, qy, data.sum(axis=2).T, norm=mpl.colors.LogNorm())
xlabel(r"Qx ($\mathrm{\AA}^{-1}$)")
ylabel(r"Qy ($\mathrm{\AA}^{-1}$)")

subplot(222)
pcolormesh(qx, qz, data.sum(axis=1).T, norm=mpl.colors.LogNorm())
xlabel(r"Qx ($\mathrm{\AA}^{-1}$)")
ylabel(r"Qz ($\mathrm{\AA}^{-1}$)")

subplot(223)
pcolormesh(qy, qz, data.sum(axis=0).T, norm=mpl.colors.LogNorm())
xlabel(r"Qy ($\mathrm{\AA}^{-1}$)")
ylabel(r"Qz ($\mathrm{\AA}^{-1}$)")
tight_layout()

#############################################################
# 2D real space maps for selected region in Q-space
qr = [0.57, 0.62, -0.20, -0.16, 3.47, 3.50]  # [xmin, xmax, ymin, ..., zmax]
x, y, data = fss.get_sxrd_for_qrange(
    qr, qconv, datadir=id01.datadir, replacedir=id01.repl_n
)
numpy.savez_compressed("output_sxrd_map.npz", x=x, y=y, data=data)


figure()
lev_exp = numpy.linspace(numpy.log10(data.min()), numpy.log10(data.max()), 100)
levs = numpy.power(10, lev_exp)
tricontourf(y, x, data, levs, norm=mpl.colors.LogNorm())
axis("scaled")
ylabel("piy (um)")
xlabel("pix (um)")
tight_layout()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2018-2023 Dominik Kriegner <dominik.kriegner@gmail.com>

from numpy import inf, linspace, mean, sqrt
from matplotlib.pylab import (
    clf,
    figure,
    legend,
    mpl,
    semilogy,
    show,
    tight_layout,
    xlabel,
    xlim,
    ylabel,
)

import xrayutilities as xu

mpl.rcParams["font.size"] = 16.0

en = "CuKa1"  # eV
resol = 0.0004  # resolution in q

sub = xu.simpack.Layer(xu.materials.Si, inf)
lay = xu.simpack.Layer(xu.materials.SiGe(0.6), 145.87, relaxation=0.0)
# pseudomorphic stack -> adjusts lattice parameters!
pls = xu.simpack.PseudomorphicStack001("pseudo", sub, lay)

H, K, L = (0, 0, 4)
qz = linspace(4.0, 7.0, 3000)

# calculate incidence angle for dynamical diffraction models
qx = sqrt(
    pls[0].material.Q(H, K, L)[0] ** 2 + pls[0].material.Q(H, K, L)[1] ** 2
)
ai = xu.simpack.coplanar_alphai(qx, qz, en)
resolai = abs(
    xu.simpack.coplanar_alphai(qx, mean(qz) + resol, en)
    - xu.simpack.coplanar_alphai(qx, mean(qz), en)
)

# kinematic multibeam diffraction model
mk = xu.simpack.KinematicalMultiBeamModel(
    pls, energy=en, surface_hkl=(0, 0, 1), resolution_width=resol
)
Imult = mk.simulate(qz, hkl=(H, K, L), refraction=True)

# general 2-beam theory based dynamical diffraction model
md = xu.simpack.DynamicalModel(pls, energy=en, resolution_width=resolai)
Idyn = md.simulate(ai, hkl=(H, K, L))

# plot of calculated intensities
figure("XU-simpack SiGe 004")
clf()
semilogy(qz, Imult, label="multibeam")
semilogy(xu.simpack.get_qz(qx, ai, en), Idyn, label="full dynamical")
legend(fontsize="small")
xlim(qz.min(), qz.max())
xlabel(r"Qz ($1/\mathrm{\AA}$)")
ylabel("Intensity (arb. u.)")
tight_layout()
show()

H, K, L = (2, 2, 4)
qz = linspace(4.0, 7.0, 3000)

# calculate incidence angle for dynamical diffraction models
qx = sqrt(
    pls[0].material.Q(H, K, L)[0] ** 2 + pls[0].material.Q(H, K, L)[1] ** 2
)
ai = xu.simpack.coplanar_alphai(qx, qz, en)
resolai = abs(
    xu.simpack.coplanar_alphai(qx, mean(qz) + resol, en)
    - xu.simpack.coplanar_alphai(qx, mean(qz), en)
)

# kinematic multibeam diffraction model
mk = xu.simpack.KinematicalMultiBeamModel(
    pls, energy=en, surface_hkl=(0, 0, 1), resolution_width=resol
)
Imult = mk.simulate(qz, hkl=(H, K, L), refraction=True)

# general 2-beam theory based dynamical diffraction model
md = xu.simpack.DynamicalModel(pls, energy=en, resolution_width=resolai)
Idyn = md.simulate(ai, hkl=(H, K, L))

# plot of calculated intensities
figure("XU-simpack SiGe 224")
clf()
semilogy(qz, Imult, label="multibeam")
semilogy(xu.simpack.get_qz(qx, ai, en), Idyn, label="full dynamical")
legend(fontsize="small")
xlim(qz.min(), qz.max())
xlabel(r"Qz ($1/\mathrm{\AA}$)")
ylabel("Intensity (arb. u.)")
tight_layout()
show()

H, K, L = (1, 1, 5)
qz = linspace(4.0, 7.0, 3000)

# calculate incidence angle for dynamical diffraction models
qx = sqrt(
    pls[0].material.Q(H, K, L)[0] ** 2 + pls[0].material.Q(H, K, L)[1] ** 2
)
ai = xu.simpack.coplanar_alphai(qx, qz, en)
resolai = abs(
    xu.simpack.coplanar_alphai(qx, mean(qz) + resol, en)
    - xu.simpack.coplanar_alphai(qx, mean(qz), en)
)

# kinematic multibeam diffraction model
mk = xu.simpack.KinematicalMultiBeamModel(
    pls, energy=en, surface_hkl=(0, 0, 1), resolution_width=resol
)
Imult = mk.simulate(qz, hkl=(H, K, L), refraction=True)

# general 2-beam theory based dynamical diffraction model
md = xu.simpack.DynamicalModel(pls, energy=en, resolution_width=resolai)
Idyn = md.simulate(ai, hkl=(H, K, L))

# plot of calculated intensities
figure("XU-simpack SiGe 115")
clf()
semilogy(qz, Imult, label="multibeam")
semilogy(xu.simpack.get_qz(qx, ai, en), Idyn, label="full dynamical")
legend(fontsize="small")
xlim(qz.min(), qz.max())
xlabel(r"Qz ($1/\mathrm{\AA}$)")
ylabel("Intensity (arb. u.)")
tight_layout()
show()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2009 Eugen Wintersberger <eugen.wintersberger@desy.de>
# Copyright (C) 2010-2025 Dominik Kriegner <dominik.kriegner@gmail.com>

from .atom import Atom

Dummy = Atom("Dummy", 0)
H = Atom("H", 1)
Hdot = Atom("H.", 1)
H1m = Atom("H1-", 1)
D = Atom("D", 1)
T = Atom("T", 1)
He = Atom("He", 2)
Li = Atom("Li", 3)
Li1p = Atom("Li1+", 3)
Be = Atom("Be", 4)
Be2p = Atom("Be2+", 4)
B = Atom("B", 5)
C = Atom("C", 6)
Cdot = Atom("C.", 6)
N = Atom("N", 7)
O = Atom("O", 8)  # noqa: E741
O1m = Atom("O1-", 8)
O2mdot = Atom("O2-.", 8)
O2m = Atom("O2-.", 8)
F = Atom("F", 9)
F1m = Atom("F1-", 9)
Ne = Atom("Ne", 10)
Na = Atom("Na", 11)
Na1p = Atom("Na1+", 11)
Mg = Atom("Mg", 12)
Mg2p = Atom("Mg2+", 12)
Al = Atom("Al", 13)
Al3p = Atom("Al3+", 13)
Si = Atom("Si", 14)
Sidot = Atom("Si.", 14)
Si4p = Atom("Si4+", 14)
P = Atom("P", 15)
S = Atom("S", 16)
Cl = Atom("Cl", 17)
Cl1m = Atom("Cl1-", 17)
Ar = Atom("Ar", 18)
K = Atom("K", 19)
K1p = Atom("K1+", 19)
Ca = Atom("Ca", 20)
Ca2p = Atom("Ca2+", 20)
Sc = Atom("Sc", 21)
Sc3p = Atom("Sc3+", 21)
Ti = Atom("Ti", 22)
Ti2p = Atom("Ti2+", 22)
Ti3p = Atom("Ti3+", 22)
Ti4p = Atom("Ti4+", 22)
V = Atom("V", 23)
V2p = Atom("V2+", 23)
V3p = Atom("V3+", 23)
V5p = Atom("V5+", 23)
Cr = Atom("Cr", 24)
Cr2p = Atom("Cr2+", 24)
Cr3p = Atom("Cr3+", 24)
Mn = Atom("Mn", 25)
Mn2p = Atom("Mn2+", 25)
Mn3p = Atom("Mn3+", 25)
Mn4p = Atom("Mn4+", 25)
Fe = Atom("Fe", 26)
Fe2p = Atom("Fe2+", 26)
Fe3p = Atom("Fe3+", 26)
Co = Atom("Co", 27)
Co2p = Atom("Co2+", 27)
Co3p = Atom("Co3+", 27)
Ni = Atom("Ni", 28)
Ni2p = Atom("Ni2+", 28)
Ni3p = Atom("Ni3+", 28)
Cu = Atom("Cu", 29)
Cu1p = Atom("Cu1+", 29)
Cu2p = Atom("Cu2+", 29)
Zn = Atom("Zn", 30)
Zn2p = Atom("Zn2+", 30)
Ga = Atom("Ga", 31)
Ga3p = Atom("Ga3+", 31)
Ge = Atom("Ge", 32)
Ge4p = Atom("Ge4+", 32)
As = Atom("As", 33)
Se = Atom("Se", 34)
Br = Atom("Br", 35)
Br1m = Atom("Br1-", 35)
Kr = Atom("Kr", 36)
Rb = Atom("Rb", 37)
Rb1p = Atom("Rb1+", 37)
Sr = Atom("Sr", 38)
Sr2p = Atom("Sr2+", 38)
Y = Atom("Y", 39)
Y3p = Atom("Y3+", 39)
Zr = Atom("Zr", 40)
Zr4p = Atom("Zr4+", 40)
Nb = Atom("Nb", 41)
Nb3p = Atom("Nb3+", 41)
Nb5p = Atom("Nb5+", 41)
Mo = Atom("Mo", 42)
Mo3p = Atom("Mo3+", 42)
Mo5p = Atom("Mo5+", 42)
Mo6p = Atom("Mo6+", 42)
Tc = Atom("Tc", 43)
Ru = Atom("Ru", 44)
Ru3p = Atom("Ru3+", 44)
Ru4p = Atom("Ru4+", 44)
Rh = Atom("Rh", 45)
Rh3p = Atom("Rh3+", 45)
Rh4p = Atom("Rh4+", 45)
Pd = Atom("Pd", 46)
Pd2p = Atom("Pd2+", 46)
Pd4p = Atom("Pd4+", 46)
Ag = Atom("Ag", 47)
Ag1p = Atom("Ag1+", 47)
Ag2p = Atom("Ag2+", 47)
Cd = Atom("Cd", 48)
Cd2p = Atom("Cd2+", 48)
In = Atom("In", 49)
In3p = Atom("In3+", 49)
Sn = Atom("Sn", 50)
Sn2p = Atom("Sn2+", 50)
Sn4p = Atom("Sn4+", 50)
Sb = Atom("Sb", 51)
Sb3p = Atom("Sb3+", 51)
Sb5p = Atom("Sb5+", 51)
Te = Atom("Te", 52)
I = Atom("I", 53)  # noqa: E741
I1m = Atom("I1-", 53)
Xe = Atom("Xe", 54)
Cs = Atom("Cs", 55)
Cs1p = Atom("Cs1+", 55)
Ba = Atom("Ba", 56)
Ba2p = Atom("Ba2+", 56)
La = Atom("La", 57)
La3p = Atom("La3+", 57)
Ce = Atom("Ce", 58)
Ce3p = Atom("Ce3+", 58)
Ce4p = Atom("Ce4+", 58)
Pr = Atom("Pr", 59)
Pr3p = Atom("Pr3+", 59)
Pr4p = Atom("Pr4+", 59)
Nd = Atom("Nd", 60)
Nd3p = Atom("Nd3+", 60)
Pm = Atom("Pm", 61)
Pm3p = Atom("Pm3+", 61)
Sm = Atom("Sm", 62)
Sm3p = Atom("Sm3+", 62)
Eu = Atom("Eu", 63)
Eu2p = Atom("Eu2+", 63)
Eu3p = Atom("Eu3+", 63)
Gd = Atom("Gd", 64)
Gd3p = Atom("Gd3+", 64)
Tb = Atom("Tb", 65)
Tb3p = Atom("Tb3+", 65)
Dy = Atom("Dy", 66)
Dy3p = Atom("Dy3+", 66)
Ho = Atom("Ho", 67)
Ho3p = Atom("Ho3+", 67)
Er = Atom("Er", 68)
Er3p = Atom("Er3+", 68)
Tm = Atom("Tm", 69)
Tm3p = Atom("Tm3+", 69)
Yb = Atom("Yb", 70)
Yb2p = Atom("Yb2+", 70)
Yb3p = Atom("Yb3+", 70)
Lu = Atom("Lu", 71)
Lu3p = Atom("Lu3+", 71)
Hf = Atom("Hf", 72)
Hf4p = Atom("Hf4+", 72)
Ta = Atom("Ta", 73)
Ta5p = Atom("Ta5+", 73)
W = Atom("W", 74)
W6p = Atom("W6+", 74)
Re = Atom("Re", 75)
Os = Atom("Os", 76)
Os4p = Atom("Os4+", 76)
Ir = Atom("Ir", 77)
Ir3p = Atom("Ir3+", 77)
Ir4p = Atom("Ir4+", 77)
Pt = Atom("Pt", 78)
Pt2p = Atom("Pt2+", 78)
Pt4p = Atom("Pt4+", 78)
Au = Atom("Au", 79)
Au1p = Atom("Au1+", 79)
Au3p = Atom("Au3+", 79)
Hg = Atom("Hg", 80)
Hg1p = Atom("Hg1+", 80)
Hg2p = Atom("Hg2+", 80)
Tl = Atom("Tl", 81)
Tl1p = Atom("Tl1+", 81)
Tl3p = Atom("Tl3+", 81)
Pb = Atom("Pb", 82)
Pb2p = Atom("Pb2+", 82)
Pb4p = Atom("Pb4+", 82)
Bi = Atom("Bi", 83)
Bi3p = Atom("Bi3+", 83)
Bi5p = Atom("Bi5+", 83)
Po = Atom("Po", 84)
At = Atom("At", 85)
Rn = Atom("Rn", 86)
Fr = Atom("Fr", 87)
Ra = Atom("Ra", 88)
Ra2p = Atom("Ra2+", 88)
Ac = Atom("Ac", 89)
Ac3p = Atom("Ac3+", 89)
Th = Atom("Th", 90)
Th4p = Atom("Th4+", 90)
Pa = Atom("Pa", 91)
U = Atom("U", 92)
U3p = Atom("U3+", 92)
U4p = Atom("U4+", 92)
U6p = Atom("U6+", 92)
Np = Atom("Np", 93)
Np3p = Atom("Np3+", 93)
Np4p = Atom("Np4+", 93)
Np6p = Atom("Np6+", 93)
Pu = Atom("Pu", 94)
Pu3p = Atom("Pu3+", 94)
Pu4p = Atom("Pu4+", 94)
Pu6p = Atom("Pu6+", 94)
Am = Atom("Am", 95)
Cm = Atom("Cm", 96)
Bk = Atom("Bk", 97)
Cf = Atom("Cf", 98)
Es = Atom("Es", 99)
Fm = Atom("Fm", 100)
Md = Atom("Md", 101)
No = Atom("No", 102)
Lr = Atom("Lr", 103)
Rf = Atom("Rf", 104)
Db = Atom("Db", 105)
Sg = Atom("Sg", 106)
Bh = Atom("Bh", 107)
Hs = Atom("Hs", 108)
Mt = Atom("Mt", 109)
Ds = Atom("Ds", 110)
Rg = Atom("Rg", 111)
Cn = Atom("Cn", 112)
Uut = Atom("Uut", 113)
Uuq = Atom("Uuq", 114)
Uup = Atom("Uup", 115)
Uuh = Atom("Uuh", 116)
Uus = Atom("Uus", 117)
Uuo = Atom("Uuo", 118)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2020 Dominik Kriegner <dominik.kriegner@gmail.com>

import itertools
import re
import unittest

import numpy
import xrayutilities as xu
from xrayutilities.materials import SymOp


class TestMaterialsSymOp(unittest.TestCase):
    @classmethod
    def setUp(cls):
        cls.x, cls.y, cls.z = numpy.random.rand(3)
        a, b, c = numpy.random.rand(3) * 2 + 4
        al, be, gam = numpy.random.rand(3) * 60 + 60
        pdict = {"a": a, "b": b, "c": c, "alpha": al, "beta": be, "gamma": gam}

        cls.lats = []
        cls.gps = []
        for sg in xu.materials.spacegrouplattice.wp.keys():
            # determine parameters for this space group
            sgnr = int(sg.split(":")[0])
            csys, nargs = xu.materials.spacegrouplattice.sgrp_sym[sgnr]
            params = xu.materials.spacegrouplattice.sgrp_params[csys][0]
            p = [eval(par, pdict) for par in params]
            # generate test lattice
            cls.lats.append(xu.materials.SGLattice(sg, *p))
            # get general Wyckoff position of space group
            wp = xu.materials.spacegrouplattice.wp
            gplabel = sorted(wp[sg], key=lambda s: int(s[:-1]))[-1]
            cls.gps.append(wp[sg][gplabel][1])

    def test_SymOp_fromxyz(self):
        """
        tests that the creation of the symmetry operations from the xyz string
        of the Wyckoff positions is reproducible/reversable
        """
        for lat, gp in zip(self.lats, self.gps):
            symopsxyz = map(lambda s: "({})".format(s.xyz()), lat.symops)
            self.assertCountEqual(symopsxyz, gp)

    def test_equivalent_hkl(self):
        hkl = numpy.random.randint(-11, 12, 3)
        for lat in self.lats:
            ehkl = numpy.unique(
                numpy.einsum("...ij,j", lat._hklsym, hkl), axis=0
            )
            ehkl = set(tuple(e) for e in ehkl)
            self.assertEqual(lat.equivalent_hkls(hkl), ehkl)

    def test_iscentrosymmetric(self):
        centrosym = list(
            itertools.chain(
                [2],
                range(10, 16),
                range(47, 75),
                range(83, 89),
                range(123, 143),
                [147, 148],
                range(162, 168),
                [175, 176],
                range(191, 195),
                range(200, 207),
                range(221, 231),
            )
        )
        for lat in self.lats:
            if lat.space_group_nr in centrosym:
                self.assertTrue(lat.iscentrosymmetric)
            else:
                self.assertFalse(lat.iscentrosymmetric)

    def test_Wyckoff_consistency(self):
        """
        tests that all Wyckoff positions are consistent with the symmetry
        operations
        """
        reint = re.compile("[0-9]+")
        pardict = {"x": self.x, "y": self.y, "z": self.z}
        wp = xu.materials.spacegrouplattice.wp
        for lat, gp in zip(self.lats, self.gps):
            # check every Wyckoff position
            for wpkey in wp[lat.space_group]:
                uniquepos = []
                poscount = int(reint.match(wpkey).group())
                thispositions = list(
                    map(
                        lambda p: SymOp.foldback(eval(p, pardict)),
                        wp[lat.space_group][wpkey][1],
                    )
                )
                pos0 = SymOp.foldback(thispositions[0])
                genpos = [s.apply(pos0) for s in lat.symops]
                uniquepos = [
                    genpos[0],
                ]
                for p in genpos:
                    considered = False
                    for u in uniquepos:
                        if numpy.allclose(p, u):
                            considered = True
                    if not considered:
                        uniquepos.append(p)
                comparepos = [
                    (t, g)
                    for t in thispositions
                    for g in uniquepos
                    if numpy.allclose(t, g)
                ]
                # check that number of Wyckoff position entries are correct
                self.assertEqual(poscount, len(thispositions))
                # check that an equal number of unique positions is produced
                self.assertEqual(poscount, len(uniquepos))
                # check that unique positions are equal to Wyckoff positions
                self.assertEqual(poscount, len(comparepos))


if __name__ == "__main__":
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015-2020 Dominik Kriegner <dominik.kriegner@gmail.com>

import os.path
import unittest

import xrayutilities as xu

xu.config.VERBOSITY = 0  # make no output during test
testfile = "seifert_map.nja.bz2"
datadir = os.path.join(os.path.dirname(__file__), "data")
fullfilename = os.path.join(datadir, testfile)


@unittest.skipIf(
    not os.path.isfile(fullfilename),
    "additional test data needed (http://xrayutilities.sf.io)",
)
class TestIO_map_NJA(unittest.TestCase):
    dshape = (601601,)
    dmax = 12.8161590
    dmin = 0.0
    motmin = 77.0
    motmax = 80.0
    dtpos = 1.8286480
    tpos = 294840

    @classmethod
    def setUpClass(cls):
        cls.mot, cls.mot2, cls.data = xu.io.getSeifert_map(
            testfile, path=datadir
        )

    def test_datashape(self):
        self.assertEqual(self.dshape, self.data.shape)

    def test_datavalues(self):
        self.assertAlmostEqual(self.motmax, self.mot.max(), places=6)
        self.assertAlmostEqual(self.motmin, self.mot.min(), places=6)
        self.assertAlmostEqual(self.dmax, self.data.max(), places=6)
        self.assertAlmostEqual(self.dmin, self.data.min(), places=6)
        self.assertAlmostEqual(self.dtpos, self.data[self.tpos], places=6)


if __name__ == "__main__":
    unittest.main()



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2012-2021 Dominik Kriegner <dominik.kriegner@gmail.com>
"""simulation subpackage of xrayutilities.

This package provides possibilities to simulate X-ray diffraction and
reflectivity curves of thin film samples. It could be extended for more
general use in future if there is demand for that.

In addition it provides a fitting routine for reflectivity data which is based
on lmfit.
"""

from .darwin_theory import (  # noqa: F401
    DarwinModel,
    DarwinModelAlGaAs001,
    DarwinModelAlloy,
    DarwinModelGaInAs001,
    DarwinModelSiGe001,
    GradedBuffer,
)
from .fit import FitModel  # noqa: F401
from .helpers import coplanar_alphaf, coplanar_alphai, get_qz  # noqa: F401
from .models import (  # noqa: F401
    DiffuseReflectivityModel,
    DynamicalModel,
    DynamicalReflectivityModel,
    KinematicalModel,
    KinematicalMultiBeamModel,
    LayerModel,
    Model,
    SimpleDynamicalCoplanarModel,
    SpecularReflectivityModel,
    effectiveDensitySlicing,
)
from .mosaicity import mosaic_analytic  # noqa: F401
from .powder import FP_profile, PowderDiffraction  # noqa: F401
from .powdermodel import (  # noqa: F401
    PowderModel,
    Rietveld_error_metrics,
    plot_powder,
)
from .smaterials import (  # noqa: F401
    CrystalStack,
    GradedLayerStack,
    Layer,
    LayerStack,
    MaterialList,
    Powder,
    PowderList,
    PseudomorphicStack001,
    PseudomorphicStack111,
    SMaterial,
)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2023 C2N - CNRS
# Author:	  Aristide Lemaître <aristide.lemaitre@c2n.upsaclay.fr>


# This is an example of the modelling and fitting of the diffraction data
# obtained on epitaxial heterostructure
# The design is as follow
#              _________________________
#                                             using the same Ga cell as
#                       GaAs 320 nm           GaAs below (same growth rate)
#               _________________________
#                                             using the same Ga & Al cells as
#                  Al_0.80GaAs 1.8 micron     Al_0.80GaAs below (same growth
#               _________________________     rate and composition)
# #
#                     GaAs 200 nm
#              _________________________
# #
#                  Al_0.80GaAs 2 micron
#               _________________________
# #
#              GaAs substrate and buffer (001)
# #
# Xray set-up: X'PERT Panalytical with
#       copper anode,focusing mirror, Ge(220)x4 monochromator,
# Ge(220)x2 triple axis detection


import xrayutilities as xu
import numpy as np
import matplotlib.pyplot as plt


# read experimental file

scan = xu.io.panalytical_xml.XRDMLFile(
    "simpack_xrd_AlGaAs.xrdml.gz", path="data"
)
om = scan.scans[0]["Omega"]
Int = scan.scans[0].int


# Omega offset
offset = -0.047636
om_off = om + offset

hkl = (0, 0, 4)


# definition of materials

GaAs_ = xu.materials.GaAs
AlAs_ = xu.materials.AlAs
AlGaAs80 = xu.materials.AlGaAs(0.20)


# definition of heterostructure layers

sub = xu.simpack.Layer(GaAs_, float("inf"))
AlGaAs80_1 = xu.simpack.Layer(AlGaAs80, 20000, relaxation=0.0)
GaAs_2 = xu.simpack.Layer(GaAs_, 2000, relaxation=0.0)
AlGaAs80_3 = xu.simpack.Layer(AlGaAs80, 18000, relaxation=0.0)
GaAs_4 = xu.simpack.Layer(GaAs_, 3200, relaxation=0.0)


# heterostructure content
heterostructure = xu.simpack.PseudomorphicStack001(
    "Hetereostructure", sub, AlGaAs80_1 + GaAs_2 + AlGaAs80_3 + GaAs_4
)

# Monochromator
thMono = np.arcsin(
    xu.wavelength("CuKa1") / (2 * xu.materials.Ge.planeDistance(2, 2, 0))
)

# Dynamical diffraction model
md = xu.simpack.DynamicalModel(
    heterostructure,
    I0=4e5,
    background=1,
    resolution_width=0.0015,
    Cmono=np.cos(2 * thMono),
    polarization="both",
)

fitmdyn = xu.simpack.FitModel(md)
fitmdyn.lmodel.set_hkl(hkl)

# fitmdyn.verbose = True
fitmdyn.plot = True


# set fit parameters

# AlGaAs80_1
# 'AlAs_0_80_GaAs_0_20__c' parameter name found using
# print(params) (see below)
fitmdyn.set_param_hint(
    "AlAs_0_80_GaAs_0_20__c", vary=True, min=AlGaAs80.a, max=AlGaAs80.a * 1.002
)
fitmdyn.set_param_hint(
    "AlAs_0_80_GaAs_0_20__thickness", vary=True, min=19800, max=20200
)

# AlGaAs80_3
# AlGaAs80_3 is grown using the same cells as AlGaAs_1 hence same composition
# and growth rate
# same composition as AlGaAs80_1
fitmdyn.set_param_hint(
    "AlAs_0_80_GaAs_0_20__1_c", vary=True, expr="AlAs_0_80_GaAs_0_20__c"
)
# 18000/20000 :  thickness ratio between layer 1 and 3
fitmdyn.set_param_hint(
    "AlAs_0_80_GaAs_0_20__1_thickness",
    vary=True,
    expr="AlAs_0_80_GaAs_0_20__thickness * 18000 / 20000",
)

# GaAs_2
fitmdyn.set_param_hint("GaAs_1_thickness", vary=True, min=1990, max=2010)

# GaAs_4
# 3200/2000 : thickness ratio between GaAs_4 and GaAs_2
fitmdyn.set_param_hint(
    "GaAs_2_thickness", vary=True, expr="GaAs_1_thickness*3200/2000"
)


fitmdyn.set_param_hint("background", vary=True, min=0, max=10)
fitmdyn.set_param_hint("I0", vary=True, min=3e5, max=5e5)
fitmdyn.set_param_hint("resolution_width", vary=True, min=0.0012, max=0.0015)

params = fitmdyn.make_params()


lmfit_kws = {}
# choose your fitting method, default 'leastsq'
# 'differential_evolution' can help to overcome local minima but is slower
# lmfit_kws['method'] = 'differential_evolution'

# option to avoid error during fit
lmfit_kws["nan_policy"] = "omit"

# launch fitting procedure
fitr = fitmdyn.fit(Int, params, om_off, lmfit_kws=lmfit_kws)


# get Al composition for perpendicular lattice parameter
qz = 2 * np.pi * hkl[2] / fitr.params["AlAs_0_80_GaAs_0_20__c"]
x_Al = AlGaAs80.ContentBsym(qz, hkl, np.array([0, 0, 1]), GaAs_.a, 0)

# print fit results
print(fitr.fit_report())
print("x_Al = ", x_Al)


# Plot results

plt.clf()

plt.semilogy(om_off, Int, "r", label="Exp data")
simdyn = fitmdyn.eval(fitr.params, x=om)
plt.semilogy(om, simdyn, "b", label="Simulation")
plt.xlabel("\u03c9(°)")
plt.ylabel("Intensity (cps)")
plt.legend()
plt.text(
    33.15,
    1000,
    "GaAs: {:.1f} nm \nGaAl$_{{{:.2f}}}$As: {:.0f} nm \nGaAs: {:.1f} nm \
         \nGaAl$_{{{:.2f}}}$As: {:.0f} nm \nSubstrate".format(
        fitr.params["GaAs_2_thickness"] / 10,
        x_Al,
        fitr.params["AlAs_0_80_GaAs_0_20__1_thickness"] / 10,
        fitr.params["GaAs_1_thickness"] / 10,
        x_Al,
        fitr.params["AlAs_0_80_GaAs_0_20__thickness"] / 10,
    ),
)
plt.title("HRXRD")
plt.show()



# -*- coding: utf-8 -*-
#
# xrayutilities documentation build configuration file, created by
# sphinx-quickstart on Sat Mar  2 11:39:01 2013.
#
# This file is execfile()d with the current directory set to its containing
# dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import re
from pathlib import Path

import sphinx_rtd_theme
from sphinx_pyproject import SphinxConfig
from xrayutilities import __version__ as xu_version

# Load configuration from pyproject.toml

config = SphinxConfig(
    Path(__file__).parent.parent.parent / "pyproject.toml",
    config_overrides = {"version": xu_version},
)

# Project information (auto-extracted from pyproject.toml)
project = config.name     # Automatically gets the project name
author = config.author    # Automatically gets the author(s)
copyright = f"%Y, {config.author}"
release = config.version  # Full version (e.g., "1.2.3")
version = config.version.split(".")[0] + "." + config.version.split(".")[1]  # Short version (e.g., "1.2")

# Add any Sphinx extension module names here, as strings. They can be
# extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.coverage',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.mathjax',
    'sphinx.ext.viewcode',
    'numpydoc',
    'rst2pdf.pdfbuilder']

intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    'matplotlib': ('https://matplotlib.org/stable', None),
    'lmfit': ('https://lmfit.github.io/lmfit-py/', None),
}
# Paths
templates_path = ["_templates"]
source_suffix = ".rst"
master_doc = "index"

# solve "toctree contains reference to nonexisting document" warnings
numpydoc_show_class_members = False

# HTML output options
html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
html_theme_options = {"logo_only": True}
html_logo = "pics/xray-logo.png"
html_favicon = "favicon.ico"
# Output file base name for HTML help builder.
htmlhelp_basename = 'xrayutilitiesdoc'

# LaTeX options
latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    'papersize': 'a4paper',

    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    'preamble': '\\usepackage{enumitem}\n\\setlistdepth{15}',
}
latex_documents = [
    ('index', 'xrayutilities.tex', u'xrayutilities Documentation',
     u'Dominik Kriegner \\and Eugen Wintersberger', 'manual'),
]



# manual page options
man_pages = [
    ('index', 'xrayutilities', u'xrayutilities Documentation',
     [u'Dominik Kriegner, Eugen Wintersberger'], 1)
]


# Texinfo options
texinfo_documents = [
    ('index',
     'xrayutilities',
     u'xrayutilities Documentation',
     u'Dominik Kriegner, Eugen Wintersberger',
     'xrayutilities',
     'a Python package for diffraction physicists',
     'Miscellaneous'),
]


# PDF output options
pdf_documents = [
    ("index", "xrayutilities", "xrayutilities", author),
]
pdf_stylesheets = ["sphinx", "onecolumn"]
pdf_compressed = True
pdf_default_dpi = 120


def process_docstring(app, what, name, obj, options, lines):
    """my additions for pre-processing the docstring"""
    # insert note block
    for i in range(len(lines)):
        lines[i] = re.sub(r'^\s*Note:', r'.. note::', lines[i])


def skip(app, what, name, obj, would_skip, options):
    """always document __init__ and __call__ functions"""
    if name in ["__init__", "__call__"]:
        return False
    return would_skip


def setup(app):
    app.connect('autodoc-process-docstring', process_docstring)
    app.connect("autodoc-skip-member", skip)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (c) 2013-2020, 2023 Dominik Kriegner <dominik.kriegner@gmail.com>

import numpy

from . import elements as e
from .heuslerlib import *  # noqa: F401, F403
from .material import (
    Crystal,
    CubicAlloy,
    CubicElasticTensor,
    HexagonalElasticTensor,
    WZTensorFromCub,
)
from .spacegrouplattice import SGLattice

# some predefined materials
# PLEASE use N/m^2 as unit for cij for newly entered material ( 1 dyn/cm^2 =
# 0.1 N/m^2 = 0.1 GPa)
# Use Kelvin as unit for the Debye temperature
# ref http://www.semiconductors.co.uk/propiviv5431.htm
C = Crystal(
    "C",
    SGLattice(
        "227:1",
        3.5668,
        atoms=[
            e.C,
        ],
        pos=[
            "8a",
        ],
    ),
    CubicElasticTensor(1.0764e12, 0.125e12, 0.577e12),
)
C_HOPG = Crystal(
    "HOPG", SGLattice(194, 2.4612, 6.7079, atoms=[e.C, e.C], pos=["2b", "2c"])
)
Si = Crystal(
    "Si",
    SGLattice(
        "227:1",
        5.43104,
        atoms=[
            e.Si,
        ],
        pos=[
            "8a",
        ],
    ),
    CubicElasticTensor(165.77e9, 63.93e9, 79.62e9),
    thetaDebye=640,
)
Ge = Crystal(
    "Ge",
    SGLattice(
        "227:1",
        5.65785,
        atoms=[
            e.Ge,
        ],
        pos=[
            "8a",
        ],
    ),
    CubicElasticTensor(128.5e9, 48.3e9, 66.8e9),
    thetaDebye=374,
)
InAs = Crystal(
    "InAs",
    SGLattice(216, 6.0583, atoms=[e.In, e.As], pos=["4a", "4c"]),
    CubicElasticTensor(8.34e10, 4.54e10, 3.95e10),
    thetaDebye=280,
)
InP = Crystal(
    "InP",
    SGLattice(216, 5.8687, atoms=[e.In, e.P], pos=["4a", "4c"]),
    CubicElasticTensor(10.11e10, 5.61e10, 4.56e10),
    thetaDebye=425,
)
InSb = Crystal(
    "InSb",
    SGLattice(216, 6.47937, atoms=[e.In, e.Sb], pos=["4a", "4c"]),
    CubicElasticTensor(6.66e10, 3.65e10, 3.02e10),
    thetaDebye=160,
)
GaP = Crystal(
    "GaP",
    SGLattice(216, 5.4505, atoms=[e.Ga, e.P], pos=["4a", "4c"]),
    CubicElasticTensor(14.05e10, 6.20e10, 7.03e10),
    thetaDebye=445,
)
GaAs = Crystal(
    "GaAs",
    SGLattice(216, 5.65325, atoms=[e.Ga, e.As], pos=["4a", "4c"]),
    CubicElasticTensor(11.9e10, 5.34e10, 5.96e10),
    thetaDebye=360,
)
# data updated from Appl. Phys. Lett. 66, 682–684 (1995)
AlAs = Crystal(
    "AlAs",
    SGLattice(216, 5.66172, atoms=[e.Al, e.As], pos=["4a", "4c"]),
    CubicElasticTensor(11.99e10, 5.75e10, 5.66e10),
    thetaDebye=446,
)
GaSb = Crystal(
    "GaSb",
    SGLattice(216, 6.09593, atoms=[e.Ga, e.Sb], pos=["4a", "4c"]),
    CubicElasticTensor(8.83e10, 4.02e10, 4.32e10),
    thetaDebye=266,
)
AlSb = Crystal(
    "AlSb",
    SGLattice(216, 6.1355, atoms=[e.Al, e.Sb], pos=["4a", "4c"]),
    CubicElasticTensor(8.769e10, 4.341e10, 4.076e10),
    thetaDebye=370,
)
# from Cryst. Growth Des. 15, 4795-4803 (2015)
GaAsWZ = Crystal(
    "GaAs(WZ)",
    SGLattice(
        186,
        3.9845,
        6.5701,
        atoms=[e.Ga, e.As],
        pos=[("2b", 0), ("2b", 3 / 8.0)],
    ),
    WZTensorFromCub(11.9e10, 5.34e10, 5.96e10),
)
GaAs4H = Crystal(
    "GaAs(4H)",
    SGLattice(
        186,
        3.9900,
        13.0964,
        atoms=[e.Ga, e.Ga, e.As, e.As],
        pos=[("2a", 0), ("2b", 1 / 4.0), ("2a", 3 / 16.0), ("2b", 7 / 16.0)],
    ),
)
# from Phys. Rev. B 88, 115315 (2013)
GaPWZ = Crystal(
    "GaP(WZ)",
    SGLattice(
        186,
        3.8419,
        6.3353,
        atoms=[e.Ga, e.P],
        pos=[("2b", 0), ("2b", 0.37385)],
    ),
    WZTensorFromCub(14.05e10, 6.20e10, 7.03e10),
)
# from Nanotechnology 22 425704 (2011)
InPWZ = Crystal(
    "InP(WZ)",
    SGLattice(
        186,
        4.1423,
        6.8013,
        atoms=[e.In, e.P],
        pos=[("2b", 0), ("2b", 3 / 8.0)],
    ),
    WZTensorFromCub(10.11e10, 5.61e10, 4.56e10),
)
# from Nano Lett., 2011, 11 (4), pp 1483-1489
InAsWZ = Crystal(
    "InAs(WZ)",
    SGLattice(
        186,
        4.2742,
        7.0250,
        atoms=[e.In, e.As],
        pos=[("2b", 0), ("2b", 3 / 8.0)],
    ),
    WZTensorFromCub(8.34e10, 4.54e10, 3.95e10),
)
InAs4H = Crystal(
    "InAs(4H)",
    SGLattice(
        186,
        4.2780,
        14.0171,
        atoms=[e.In, e.In, e.As, e.As],
        pos=[("2a", 0), ("2b", 1 / 4.0), ("2a", 3 / 16.0), ("2b", 7 / 16.0)],
    ),
)
InSbWZ = Crystal(
    "InSb(WZ)",
    SGLattice(
        186,
        4.5712,
        7.5221,
        atoms=[e.In, e.Sb],
        pos=[("2b", 0), ("2b", 3 / 8.0)],
    ),
    WZTensorFromCub(6.66e10, 3.65e10, 3.02e10),
)
InSb4H = Crystal(
    "InSb(4H)",
    SGLattice(
        186,
        4.5753,
        15.0057,
        atoms=[e.In, e.In, e.Sb, e.Sb],
        pos=[("2a", 0), ("2b", 1 / 4.0), ("2a", 3 / 16.0), ("2b", 7 / 16.0)],
    ),
)

# ? Unit of elastic constants for CdTe, PbTe, PbSe ?
PbTe = Crystal(
    "PbTe",
    SGLattice(225, 6.464, atoms=[e.Pb, e.Te], pos=["4a", "4b"]),
    CubicElasticTensor(93.6, 7.7, 13.4),
)
PbSe = Crystal(
    "PbSe",
    SGLattice(225, 6.128, atoms=[e.Pb, e.Se], pos=["4a", "4b"]),
    CubicElasticTensor(123.7, 19.3, 15.9),
)
CdTe = Crystal(
    "CdTe",
    SGLattice(216, 6.482, atoms=[e.Cd, e.Te], pos=["4a", "4c"]),
    CubicElasticTensor(53.5, 36.7, 19.9),
)
CdSe = Crystal(
    "CdSe",
    SGLattice(
        186,
        4.5712,
        7.5221,
        atoms=[e.In, e.Sb],
        pos=[("2b", 0), ("2b", 3 / 8.0)],
    ),
    HexagonalElasticTensor(7.490e10, 4.609e10, 3.926e10, 8.451e10, 1.315e10),
)
CdSe_ZB = Crystal(
    "CdSe(ZB)", SGLattice(216, 6.052, atoms=[e.Cd, e.Se], pos=["4a", "4c"])
)
HgSe = Crystal(
    "HgSe",
    SGLattice(216, 6.085, atoms=[e.Hg, e.Se], pos=["4a", "4c"]),
    CubicElasticTensor(6.1e10, 4.4e10, 2.2e10),
)
NaCl = Crystal(
    "NaCl", SGLattice(225, 5.6402, atoms=[e.Na, e.Cl], pos=["4a", "4b"])
)
MgO = Crystal(
    "MgO", SGLattice(225, 4.212, atoms=[e.Mg, e.O], pos=["4a", "4b"])
)
GaN = Crystal(
    "GaN",
    SGLattice(
        186, 3.189, 5.186, atoms=[e.Ga, e.N], pos=[("2b", 0), ("2b", 3 / 8.0)]
    ),
    HexagonalElasticTensor(390.0e9, 145.0e9, 106.0e9, 398.0e9, 105.0e9),
    thetaDebye=600,
)
BaF2 = Crystal(
    "BaF2", SGLattice(225, 6.2001, atoms=[e.Ba, e.F], pos=["4a", "8c"])
)
SrF2 = Crystal(
    "SrF2", SGLattice(225, 5.8007, atoms=[e.Sr, e.F], pos=["4a", "8c"])
)
CaF2 = Crystal(
    "CaF2", SGLattice(225, 5.4631, atoms=[e.Ca, e.F], pos=["4a", "8c"])
)
MnO = Crystal(
    "MnO", SGLattice(225, 4.444, atoms=[e.Mn, e.O], pos=["4a", "4b"])
)
MnTe = Crystal(
    "MnTe",
    SGLattice(194, 4.1429, 6.7031, atoms=[e.Mn, e.Te], pos=["2a", "2c"]),
)
GeTe = Crystal(
    "GeTe",
    SGLattice(
        "160:R",
        5.996,
        88.18,
        atoms=[e.Ge, e.Ge, e.Te, e.Te],
        pos=[
            ("1a", -0.237),
            ("3b", (0.5 - 0.237, -0.237)),
            ("1a", 0.237),
            ("3b", (0.5 + 0.237, +0.237)),
        ],
    ),
)
SnTe = Crystal(
    "SnTe", SGLattice(225, 6.3268, atoms=[e.Sn, e.Te], pos=["4a", "4b"])
)
Al = Crystal(
    "Al",
    SGLattice(
        225,
        4.04958,
        atoms=[
            e.Al,
        ],
        pos=[
            "4a",
        ],
    ),
)
Au = Crystal(
    "Au",
    SGLattice(
        225,
        4.0782,
        atoms=[
            e.Au,
        ],
        pos=[
            "4a",
        ],
    ),
)
Fe = Crystal(
    "Fe",
    SGLattice(
        229,
        2.8665,
        atoms=[
            e.Fe,
        ],
        pos=[
            "2a",
        ],
    ),
)
Cr = Crystal(
    "Cr",
    SGLattice(
        229,
        2.910,
        atoms=[
            e.Cr,
        ],
        pos=[
            "2a",
        ],
    ),
)
Co = Crystal(
    "Co",
    SGLattice(
        194,
        2.5071,
        4.0695,
        atoms=[
            e.Co,
        ],
        pos=[
            "2c",
        ],
    ),
)
Ti = Crystal(
    "Ti",
    SGLattice(
        194,
        2.9508,
        4.6855,
        atoms=[
            e.Ti,
        ],
        pos=[
            "2c",
        ],
    ),
)
Mo = Crystal(
    "Mo",
    SGLattice(
        229,
        3.147,
        atoms=[
            e.Mo,
        ],
        pos=[
            "2a",
        ],
    ),
)
Ru = Crystal(
    "Ru",
    SGLattice(
        194,
        2.7059,
        4.2815,
        atoms=[
            e.Ru,
        ],
        pos=[
            "2c",
        ],
    ),
)
Rh = Crystal(
    "Rh",
    SGLattice(
        225,
        3.8034,
        atoms=[
            e.Rh,
        ],
        pos=[
            "4a",
        ],
    ),
)
V = Crystal(
    "V",
    SGLattice(
        229,
        3.024,
        atoms=[
            e.V,
        ],
        pos=[
            "2a",
        ],
    ),
)
Ta = Crystal(
    "Ta",
    SGLattice(
        229,
        3.306,
        atoms=[
            e.Ta,
        ],
        pos=[
            "2a",
        ],
    ),
)
Nb = Crystal(
    "Nb",
    SGLattice(
        229,
        3.3004,
        atoms=[
            e.Nb,
        ],
        pos=[
            "2a",
        ],
    ),
)
Pt = Crystal(
    "Pt",
    SGLattice(
        225,
        3.9242,
        atoms=[
            e.Pt,
        ],
        pos=[
            "4a",
        ],
    ),
)
W = Crystal(
    "W",
    SGLattice(
        229,
        3.1652,
        atoms=[
            e.W,
        ],
        pos=[
            "2a",
        ],
    ),
)
Ag2Se = Crystal(
    "Ag2Se",
    SGLattice(
        19,
        4.333,
        7.062,
        7.764,
        atoms=[e.Ag, e.Ag, e.Se],
        pos=[
            ("4a", (0.107, 0.369, 0.456)),
            ("4a", (0.728, 0.029, 0.361)),
            ("4a", (0.358, 0.235, 0.149)),
        ],
    ),
)
TiO2 = Crystal(
    "TiO2",
    SGLattice(136, 4.59, 2.96, atoms=[e.Ti, e.O], pos=["2a", ("4f", 0.30479)]),
)
MnO2 = Crystal(
    "MnO2",
    SGLattice(136, 4.40, 2.87, atoms=[e.Mn, e.O], pos=["2a", ("4f", 0.30479)]),
)
VO2_Rutile = Crystal(
    "VO2",
    SGLattice(136, 4.55, 2.88, atoms=[e.V, e.O], pos=["2a", ("4f", 0.305)]),
)
VO2_Baddeleyite = Crystal(
    "VO2",
    SGLattice(
        14,
        5.75,
        5.42,
        5.38,
        122.6,
        atoms=[e.V, e.O, e.O],
        pos=[
            ("4e", (0.242, 0.975, 0.025)),
            ("4e", (0.1, 0.21, 0.20)),
            ("4e", (0.39, 0.69, 0.29)),
        ],
    ),
)
SiO2 = Crystal(
    "SiO2",
    SGLattice(
        154,
        4.916,
        5.4054,
        atoms=[e.Si, e.O],
        pos=[("3a", 0.46970), ("6c", (0.41350, 0.26690, 0.11910 + 2 / 3.0))],
    ),
)
In = Crystal(
    "In",
    SGLattice(
        139,
        3.2523,
        4.9461,
        atoms=[
            e.In,
        ],
        pos=[
            "2a",
        ],
    ),
)
Sb = Crystal(
    "Sb",
    SGLattice(
        "166:H",
        4.307,
        11.273,
        atoms=[
            e.Sb,
        ],
        pos=[
            ("6c", 0.23349),
        ],
    ),
)
Sn = Crystal(
    "Sn",
    SGLattice(
        "141:1",
        5.8197,
        3.17488,
        atoms=[
            e.Sn,
        ],
        pos=[
            "4a",
        ],
    ),
)
Ag = Crystal(
    "Ag",
    SGLattice(
        225,
        4.0853,
        atoms=[
            e.Ag,
        ],
        pos=[
            "4a",
        ],
    ),
)
SnAlpha = Crystal(
    "Sn-alpha",
    SGLattice(
        "227:1",
        6.4912,
        atoms=[
            e.Sn,
        ],
        pos=[
            "8a",
        ],
    ),
)
Cu = Crystal(
    "Cu",
    SGLattice(
        225,
        3.61496,
        atoms=[
            e.Cu,
        ],
        pos=[
            "4a",
        ],
    ),
)
CaTiO3 = Crystal(
    "CaTiO3",
    SGLattice(221, 3.795, atoms=[e.Ca, e.Ti, e.O], pos=["1a", "1b", "3c"]),
)
SrTiO3 = Crystal(
    "SrTiO3",
    SGLattice(221, 3.905, atoms=[e.Sr, e.Ti, e.O], pos=["1a", "1b", "3c"]),
)
BaTiO3 = Crystal(
    "BaTiO3",
    SGLattice(
        99,
        3.992,
        4.036,
        atoms=[e.Ba, e.Ti, e.O, e.O],
        pos=[("1a", 1.000), ("1b", 0.5274), ("1b", 0.9993), ("2c", 0.5125)],
    ),
)
# BiFeO3 = Crystal("BiFeO3", SGLattice())
# BiFeO3 = Crystal(
#    "BiFeO3",
#    lattice.PerovskiteTypeRhombohedral(elements.Bi, elements.Fe, elements.O,
#                                       3.965, 89.3))
FeO = Crystal(
    "FeO", SGLattice(225, 4.332, atoms=[e.Fe, e.O], pos=["4a", "4b"])
)
CoO = Crystal(
    "CoO", SGLattice(225, 4.214, atoms=[e.Co, e.O], pos=["4a", "4b"])
)
Fe3O4 = Crystal(
    "Fe3O4",
    SGLattice(
        "227:2",
        8.3958,
        atoms=[e.Fe, e.Fe, e.O],
        pos=["8a", "16d", ("32e", 0.255)],
    ),
)
Co3O4 = Crystal(
    "Co3O4",
    SGLattice(
        "227:2",
        8.0821,
        atoms=[e.Co, e.Co, e.O],
        pos=["8a", "16d", ("32e", 0.255)],
    ),
)
FeRh = Crystal(
    "FeRh", SGLattice(221, 2.993, atoms=[e.Fe, e.Rh], pos=["1a", "1b"])
)
Ir20Mn80 = Crystal(
    "Ir20Mn80",
    SGLattice(
        225, 3.780, atoms=[e.Ir, e.Mn], pos=["4a", "4a"], occ=[0.2, 0.8]
    ),
)
CoFe = Crystal(
    "CoFe", SGLattice(221, 2.8508, atoms=[e.Co, e.Fe], pos=["1a", "1b"])
)
CoGa = Crystal(
    "CoGa", SGLattice(221, 2.883, atoms=[e.Co, e.Ga], pos=["1a", "1b"])
)
LaB6 = Crystal(
    "LaB6",
    SGLattice(221, 4.15692, atoms=[e.La, e.B], pos=["1a", ("6f", 0.19750)]),
)
Al2O3 = Crystal(
    "Al2O3",
    SGLattice(
        "167:H",
        4.7602,
        12.9933,
        atoms=[e.Al, e.O],
        pos=[("12c", 0.35216), ("18e", 0.30624)],
    ),
)
CuMnAs = Crystal(
    "CuMnAs",
    SGLattice(
        "129:2",
        3.8200,
        6.3180,
        atoms=[e.Cu, e.Mn, e.As],
        pos=["2b", ("2c", -0.8300), ("2c", -0.2347)],
        occ=[1, 0.86, 0.96],
        b=[3.5531, 1.8950, 1.2633],
    ),
)
Mn3Ge_cub = Crystal(
    "Mn3Ge (cub)",
    SGLattice("221", 3.8019, atoms=[e.Mn, e.Ge], pos=["3c", "1a"]),
)
Mn3Ge = Crystal(
    "Mn3Ge (hex)",
    SGLattice(
        "194", 5.34, 4.31, atoms=[e.Mn, e.Ge], pos=[("6h", 1 / 6.0), "2d"]
    ),
)
Pt3Cr = Crystal(
    "Pt3Cr", SGLattice("221", 3.876, atoms=[e.Pt, e.Cr], pos=["3c", "1a"])
)
TiN = Crystal(
    "TiN", SGLattice(225, 4.235, atoms=[e.Ti, e.N], pos=["4a", "4b"])
)


# Alloys with special properties
class SiGe(CubicAlloy):
    def __init__(self, x):
        """
        Si_{1-x} Ge_x cubic compound
        """
        super().__init__(Si, Ge, x)

    @staticmethod
    def lattice_const_AB(latA, latB, x):
        """
        method to calculate the lattice parameter of the SiGe alloy with
        composition Si_{1-x}Ge_x
        """
        return latA + (0.2 * x + 0.027 * x**2) * (
            latB - latA
        ) / numpy.linalg.norm(latB - latA)


class AlGaAs(CubicAlloy):
    def __init__(self, x):
        """
        Al_{1-x} Ga_x As cubic compound
        """
        super().__init__(AlAs, GaAs, x)



# This file is part of xrayutilities.
#
# xrayutilities is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2009-2010 Eugen Wintersberger <eugen.wintersberger@desy.de>
# Copyright (C) 2009-2019 Dominik Kriegner <dominik.kriegner@gmail.com>

from .cbf import CBFDirectory, CBFFile  # noqa: F401
from .desy_tty08 import gettty08_scan, tty08File  # noqa: F401
from .edf import EDFDirectory, EDFFile  # noqa: F401
from .fastscan import FastScan, FastScanCCD, FastScanSeries  # noqa: F401
from .helper import xu_h5open, xu_open  # noqa: F401
from .ill_numor import numor_scan, numorFile  # noqa: F401
from .imagereader import (  # noqa: F401
    ImageReader,
    PerkinElmer,
    Pilatus100K,
    RoperCCD,
    TIFFRead,
    get_tiff,
)
from .panalytical_xml import (  # noqa: F401
    XRDMLFile,
    getxrdml_map,
    getxrdml_scan,
)
from .pdcif import pdCIF, pdESG  # noqa: F401
from .rigaku_ras import RASFile, RASScan, getras_scan  # noqa: F401

# parser for the alignment log file of the rotating anode
from .rotanode_alignment import RA_Alignment  # noqa: F401
from .seifert import (  # noqa: F401
    SeifertMultiScan,
    SeifertScan,
    getSeifert_map,
)
from .spec import (  # noqa: F401
    SPECFile,
    SPECLog,
    SPECScan,
    geth5_scan,
    getspec_scan,
)
from .spectra import SPECTRAFile, geth5_spectra_map  # noqa: F401
